#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "qdebug.h"
#include <QtGui>

#include "TFMatrix.h"
#include "transform.h"
#include "axis.h"
#include "PlotControl.h"
#include "GnuPlot.h"
#include "ScatterPlot.h"

#include <QtXml>


using namespace std;


//------------------------------------------------------------------//
// コンストラクタ
//------------------------------------------------------------------//
MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);


    //初期画面表示（implus）
    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("ImplusResponse List File Create Template");
    ui->stackedWidget->setCurrentIndex(1);
    ui->stackedWidget_2->setCurrentIndex(1);




   //-----------------------------------------------------//
    // ファイル名テンプレートの初期値設定
    //-----------------------------------------------------//
    m_kind = RUN_IMP;
    m_run  = START;

    setInit(RUN_IMP);

    QDir dir; // 初期値の設定
    QString path;

    //-----------------------------------------------//
    // TSP LIST Template 画面
    //-----------------------------------------------//
    //TSPリストファイル
    path = QString("%1/D{AZIMUTH:%03d}_E{ELEVATION:%03d}_R100.wav").arg(dir.homePath());
    ui->tsp_filename->setText(path);


    // オリジナルインパルス
    QString app_dir = QCoreApplication::applicationDirPath();
    path = QString("%1/%2/%3").arg(app_dir).arg(HARK_TOOLS_DIR.c_str()).arg(IMPULSE_ORG.c_str());
    ui->tsp_org_impfile->setText(path);



    //-----------------------------------------------//
    // 定位伝達関数 画面
    //-----------------------------------------------//
    path = QString("%1/YOUR_PATH/MICARY-LocationFile").arg(dir.homePath());
    ui->m2pg_mic_file->setText(path);

    path = QString("%1/YOUR_PATH/ImpulseResponse-ListFile or TSP-ListFile").arg(dir.homePath());
    ui->m2pg_list_file->setText(path);



    //-----------------------------------------------//
    // 分離伝達関数 画面
    //-----------------------------------------------//
    path = QString("%1/YOUR_PATH/MICARY-LocationFile").arg(dir.homePath());
    ui->gtf_mic_file->setText(path);

    path = QString("%1/YOUR_PATH/ImpulseResponse-ListFile or TSP-ListFile").arg(dir.homePath());
    ui->gtf_list_file->setText(path);


    connect(&m_Process, SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(ProcFinished(int, QProcess::ExitStatus)));
    connect(&m_Process, SIGNAL(readyReadStandardOutput())          ,this, SLOT(ProcOutput()));
    connect(&m_Process, SIGNAL(readyReadStandardError()),           this, SLOT(ProcError()));


    //初期black画面
    ui->title_label_2->hide();
    ui->title_label->setGeometry(10,10,1191,701);
    ui->title_label->setStyleSheet("background-color:black");
    ui->title_label->setFocus();

    QTimer::singleShot( 1500, this, SLOT(hark_strat()) );

}



MainWindow::~MainWindow()
{

    delete   m_plotctl[DISP_srcM2PGout].harkio ;
    delete   m_plotctl[DISP_scrGTFout].harkio ;

    delete ui;

}








//---------------------------------------------------//
// IMPLUS ボタン押下時の処理
//---------------------------------------------------//
void MainWindow::on_Button_IMP_clicked()
{
    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("ImpulusResponse List File Template Create");

    setRunStatus(RUN_IMP);//set status
}



//---------------------------------------------------//
// TSP ボタン押下時の処理
//---------------------------------------------------//
void MainWindow::on_Button_TSP_clicked()
{
    ui->tabWidget_2->setCurrentIndex(0);
    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("TSP List File Template Create");
    setRunStatus(RUN_TSP);//set status
}



//---------------------------------------------------//
// MIC ボタン押下時の処理
//---------------------------------------------------//
void MainWindow::on_Button_MIC_clicked()
{

    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("MICARRAY Location File Template Create");

    setRunStatus(RUN_MIC);//set status

}




//---------------------------------------------------//
// NOISE ボタン押下時の処理
//---------------------------------------------------//
void MainWindow::on_Button_NOISE_clicked()
{

    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("NOISE Location File Template Create");

    setRunStatus(RUN_NOISE);//set status

}





//---------------------------------------------------//
// Localization ボタンを押した場合
//---------------------------------------------------//
void MainWindow::on_button_Localization_clicked()
{

    m_LastPath = ""; //LastPath clear
    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("Localization Transfer Function");

    setRunStatus(RUN_MUSIC);//set status

}






//---------------------------------------------------//
// Separationボタンをクリックした場合
//---------------------------------------------------//
void MainWindow::on_button_Separation_clicked()
{

    m_LastPath = ""; //LastPath clear
    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
    ui->top_label->setText("Separation Transfer Function");

    setRunStatus(RUN_GTF);//set status

}





//---------------------------------------------------//
// harktool4開始時の処理
//---------------------------------------------------//
void MainWindow::hark_strat()
{
    ui->title_label->hide();
    setRunStatus(RUN_IMP);//set status

    MainWindow::ButtonClearFocus();
}




//---------------------------------------------------//
//　情報ダイアログ
//---------------------------------------------------//
void MainWindow::THROW_INFO(QString msg)
{

    QMessageBox  msgBox;
    msgBox.setIcon(QMessageBox::Information);
    msgBox.setTextFormat(Qt::RichText);
    msgBox.setInformativeText(msg);
    msgBox.exec();

}




//---------------------------------------------------//
//　エラーダイアログ
//---------------------------------------------------//
void MainWindow::THROW_CRITICAL(QString msg)
{

    QMessageBox  msgBox;
    msgBox.setIcon(QMessageBox::Critical);
    msgBox.setText(msg);
    msgBox.exec();
}



//---------------------------------------------------//
//　メッセージダイアログ
//---------------------------------------------------//
void MainWindow::InfoMsgBox(QString msg)
{

    QMessageBox  msgBox;
    msgBox.setIcon(QMessageBox::Information);
    msgBox.setText(msg);
    msgBox.exec();

}




//---------------------------------------------------//
//　メッセージダイアログ 戻り値あり
//---------------------------------------------------//
int MainWindow::RetInfoMsgBox(QString msg)
{

    QMessageBox  msgBox;
    msgBox.setIcon(QMessageBox::Information);
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    msgBox.setText(msg);
    int ret = msgBox.exec();

    return ret;
}





//---------------------------------------------------//
// harktool4_cuiプロセス起動失敗時のエラーメッセージ
//---------------------------------------------------//
void MainWindow::ErrorMsg()
{

//    qDebug() << "ErrorMsg start=" << m_run;

    if(MainWindow::m_run != RUNNING)
    {
        setCheckLabel_NG();
    }

}




//---------------------------------------------------//
// harktool4_cuiプロセス起動失敗時のエラーメッセージ
//---------------------------------------------------//
void MainWindow::setCheckLabel_NG()
{
    ui->check_label->setAlignment(Qt::AlignCenter); //label 中央表示
    ui->check_label->setStyleSheet("color:red");    // 色:red
    ui->check_label->setFont( QFont("gothic", 14, QFont::Bold) );//font gothic
    ui->check_label->setText("ERROR");
}




//---------------------------------------------------//
// プログレスバー表示画面のlabel表示
//---------------------------------------------------//
void MainWindow::setCheckLabel()
{
    ui->check_label->setAlignment(Qt::AlignCenter); //label 中央表示
    ui->check_label->setStyleSheet("color:yellow");  // 色:red
    ui->check_label->setFont( QFont("gothic", 14, QFont::Bold) );//font gothic
    ui->check_label->setText("<<<  Now Checking The Process  >>>");
}





//---------------------------------------------------//
// creat ボタンを押したとき
//---------------------------------------------------//
void MainWindow::on_button_Create_clicked()
{

    qDebug() << "current Index =" << ui->stackedWidget->currentIndex();


    //現在表示されているページ番号
    switch( MainWindow::m_in_page_no)
    {
    case DISP_scrIMPtemplate:
        IMPcreateTemplate();
        break;

    case DISP_scrTSPtemplate:
        TSPcreateTemplate();
        break;
    case DISP_scrMICtemplate:
        MICcreateTemplate();
        break;

    case DISP_scrNOISEtemplate:
        NOISEcreateTemplate();
        break;

   case DISP_scrM2PG://定位伝達関数
       M2PGcreate();
        break;

    case DISP_scrGTF: //separation
        GTFcreate();
        break;

    }
}





//---------------------------------------------------//
// TSPファイル名テンプレートの選択ボタンを押したとき
//---------------------------------------------------//
void MainWindow::on_tsp_file_button_clicked()
{

    QString fname = QFileDialog::getOpenFileName(this);
    ui->tsp_filename->setText(fname);

    qDebug() << fname.toStdString().c_str();
}










//---------------------------------------------------//
//  TSP 球面座標
//---------------------------------------------------//
void MainWindow::on_tsp_spherical_clicked()
{

    //高さを無効にする
    ui->tsp_label_height->setEnabled(0);
    ui->tsp_label_heightint->setEnabled(0);
    ui->tsp_heightfrom->setEnabled(0);
    ui->tsp_heightto->setEnabled(0);
    ui->tsp_heightint->setEnabled(0);


    //仰角を有効にする
    ui->tsp_label_elv->setEnabled(1);
    ui->tsp_label_elvint->setEnabled(1);
    ui->tsp_elvfrom->setEnabled(1);
    ui->tsp_elvto->setEnabled(1);
    ui->tsp_elvint->setEnabled(1);


    //-------------------------------------//
    // 初期値の設定
    //-------------------------------------//
    QDir dir;
    QString path;

    //TSPリストファイル
    path = QString("%1/D{AZIMUTH:%03d}_E{ELEVATION:%03d}_R100.wav").arg(dir.homePath());
    ui->tsp_filename->setText(path);


}





//---------------------------------------------------//
//  TSP 円柱座標
//---------------------------------------------------//
void MainWindow::on_tsp_cylindrical_clicked()
{

    //高さを有効にする
    ui->tsp_label_height->setEnabled(1);
    ui->tsp_label_heightint->setEnabled(1);
    ui->tsp_heightfrom->setEnabled(1);
    ui->tsp_heightto->setEnabled(1);
    ui->tsp_heightint->setEnabled(1);


    //仰角を無効にする
    ui->tsp_label_elv->setEnabled(0);
    ui->tsp_label_elvint->setEnabled(0);
    ui->tsp_elvfrom->setEnabled(0);
    ui->tsp_elvto->setEnabled(0);
    ui->tsp_elvint->setEnabled(0);


    //-------------------------------------//
    // 初期値の設定
    //-------------------------------------//
    QDir dir;
    QString path;

    //TSPリストファイル
    path = QString("%1/D{AZIMUTH:%03d}_H{HEIGHT:%03d}_R100.wav").arg(dir.homePath());
    ui->tsp_filename->setText(path);



}




//---------------------------------------------------//
// TSP　元インパルスファイルの選択ボタンを押したとき
//---------------------------------------------------//
void MainWindow::on_tsp_org_Button_clicked()
{
    QString fname = QFileDialog::getOpenFileName(this);
    ui->tsp_org_impfile->setText(fname);
    qDebug() << fname.toStdString().c_str();
}






//---------------------------------------------------//
// impluse 球座標選択時
//---------------------------------------------------//
void MainWindow::on_imp_spherical_clicked()
{

    //高さを無効にする
    ui->label_height->setEnabled(0);
    ui->label_heightint->setEnabled(0);
    ui->imp_heightfrom->setEnabled(0);
    ui->imp_heightto->setEnabled(0);
    ui->imp_heightint->setEnabled(0);


    //仰角を有効にする
    ui->label_elv->setEnabled(1);
    ui->label_elvint->setEnabled(1);
    ui->imp_elvfrom->setEnabled(1);
    ui->imp_elvto->setEnabled(1);
    ui->imp_elvint->setEnabled(1);

    // インパルス応答リストファイル
    QDir dir;
    QString path;
    path = QString("%1/D{AZIMUTH:%03d}_E{ELEVATION:%03d}_R100_ch{CHANNEL:%02d}.flt").arg(dir.homePath());
    ui->imp_filename->setText(path);

}



//---------------------------------------------------//
// IMPファイル名テンプレートの選択ボタンを押したとき
//---------------------------------------------------//
void MainWindow::on_imp_file_button_clicked()
{
    QString fname = QFileDialog::getOpenFileName(this);
    ui->imp_filename->setText(fname);
    qDebug() << fname.toStdString().c_str();
}





//---------------------------------------------------//
// impluse 円柱座標選択時
//---------------------------------------------------//
void MainWindow::on_imp_cylindrical_clicked()
{

    //高さを有効にする
    ui->label_height->setEnabled(1);
    ui->label_heightint->setEnabled(1);
    ui->imp_heightfrom->setEnabled(1);
    ui->imp_heightto->setEnabled(1);
    ui->imp_heightint->setEnabled(1);


    //仰角を無効にする
    ui->label_elv->setEnabled(0);
    ui->label_elvint->setEnabled(0);
    ui->imp_elvfrom->setEnabled(0);
    ui->imp_elvto->setEnabled(0);
    ui->imp_elvint->setEnabled(0);

    // インパルス応答リストファイル
    QDir dir;
    QString path;
    path = QString("%1/D{AZIMUTH:%03d}_H{HEIGHT:%03d}_R100_ch{CHANNEL:%02d}.flt").arg(dir.homePath());
    ui->imp_filename->setText(path);

}











//---------------------------------------------------//
// mic 球座標選択時
//---------------------------------------------------//
void MainWindow::on_mic_spherical_clicked()
{

    //高さを無効にする
    ui->mic_label_height->setEnabled(0);
    ui->mic_label_heightint->setEnabled(0);
    ui->mic_heightfrom->setEnabled(0);
    ui->mic_heightto->setEnabled(0);
    ui->mic_heightint->setEnabled(0);


    //仰角を有効にする
    ui->mic_label_elv->setEnabled(1);
    ui->mic_label_elvint->setEnabled(1);
    ui->mic_elvfrom->setEnabled(1);
    ui->mic_elvto->setEnabled(1);
    ui->mic_elvint->setEnabled(1);

}





//---------------------------------------------------//
// mic 円柱座標選択時
//---------------------------------------------------//
void MainWindow::on_mic_cylindrical_clicked()
{

    //高さを有効にする
    ui->mic_label_height->setEnabled(1);
    ui->mic_label_heightint->setEnabled(1);
    ui->mic_heightfrom->setEnabled(1);
    ui->mic_heightto->setEnabled(1);
    ui->mic_heightint->setEnabled(1);


    //仰角を無効にする
    ui->mic_label_elv->setEnabled(0);
    ui->mic_label_elvint->setEnabled(0);
    ui->mic_elvfrom->setEnabled(0);
    ui->mic_elvto->setEnabled(0);
    ui->mic_elvint->setEnabled(0);

}






//---------------------------------------------------//
// 定位伝達関数 MicArrayLocatin file 選択ボタン
//---------------------------------------------------//
void MainWindow::on_m2pg_file_Button_clicked()
{
    QString fname;

    if(m_LastPath == "")
    {
        // Adds filter for XML files
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "MicArray Location File",
                                                ".",
                                                tr("XML Files(*.xml)")
                                            );
    }
    else
    {
        // Adds filter for XML files
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "MicArray Location File",
                                                m_LastPath,
                                                tr("XML Files(*.xml)")
                                            );
    }

    ui->m2pg_mic_file->setText(fname);
    m_LastPath = fname; //最後のパス格納

    qDebug() << fname.toStdString().c_str();
}


//---------------------------------------------------//
// 定位伝達関数 TSP/IR-ListFile 選択ボタン
//---------------------------------------------------//
void MainWindow::on_m2pg_list_Button_clicked()
{

    QString fname;

    if(m_LastPath == "")
    {
        // Adds filter for XML files
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "TSP/IR-ListFile",
                                                ".",
                                                tr("XML Files(*.xml)")
                                            );
    }
    else
    {
        // Adds filter for XML files
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "TSP/IR-ListFile",
                                                m_LastPath,
                                                tr("XML Files(*.xml)")
                                            );
    }

    ui->m2pg_list_file->setText(fname);
    m_LastPath = fname; //最後のパス格納

    qDebug() << fname.toStdString().c_str();
}





//---------------------------------------------------//
// テンポラリや設定ファイルを置くディレクトリ
//---------------------------------------------------//
std::string MainWindow::harktoolDir()
{

    QDir dir;
    QString path;

    path = dir.homePath();
    path = path + "/" + TEMPORARY_PATH.c_str();
    dir.mkdir(path);

    qDebug() << "harktoolDir=" << path.toStdString().c_str();
    return path.toStdString();
}









//---------------------------------------------------//
//  M2PG 表示画面のセルがクリックされたときの処理
//---------------------------------------------------//
void MainWindow::on_m2pg_table_cellDoubleClicked(int row, int column)
{
    try
    {
        qDebug() << "QTableWidget:" << "row=" << row << "column=" << column ;

        ui->m2pg_table->setEditTriggers(QAbstractItemView::NoEditTriggers);

        int text_check = ui->m2pg_table->item(3,1)->text().isEmpty();

        //データが表示されていない場合
        if( text_check == TRUE)
        {
            return;
        }

        std::vector<std::vector<double> > lists;

        switch(row)
        {
        //-----------------------------------------//
        //  Mic POSITION
        //-----------------------------------------//
        case DISP_M2PG_MIC_POSITION:
            {
                harkio::FMT *fmt =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::FMT> ();
                harkio::FMT::MIC_LIST_TYPE *mic = fmt->getMic();

                // M2PG::SRC_LIST_TYPE から listへ変換する
                for (uint i = 0; i < (*mic).size(); i++)
                {
                    std::vector<double> xyz(3);
                    xyz[0] = (*mic)[i].x;
                    xyz[1] = (*mic)[i].y;
                    xyz[2] = (*mic)[i].z;
                    lists.push_back(xyz);
                }
                // gnuplotで表示する
                ScatterPlot graph(&lists);
                graph.setTitle("Microphone Position");
                graph.setStyle(PlotStyle_POINTS);
                graph.plot(harktoolDir());
                break;
            }

        //-----------------------------------------//
        //  SOURCE_LOCATION
        //-----------------------------------------//
        case DISP_M2PG_SOURCE_LOCATION:
        {
            harkio::M2PG *m2pg = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::M2PG> ();
            harkio::M2PG::SRC_LIST_TYPE *src = m2pg->getSrc();

            for ( uint i= 0; i < (*src).size() ; i++)
            {
                for (uint j = 0; j < (*src)[0].size(); j++)
                {
                    std::vector<double> xyz(3);
                    xyz[0] = (*src)[i][j].x;
                    xyz[1] = (*src)[i][j].y;
                    xyz[2] = (*src)[i][j].z;
                    lists.push_back(xyz);
                }
           }

            // gnuplotで表示する
            ScatterPlot graph(&lists);
            graph.setTitle("Source Position");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());
            break;
        }

        //-----------------------------------------//
        //trance func のグラフ表示
        //-----------------------------------------//
        case DISP_M2PG_TRANSFER_FUNCTION:
        {

            //入力画面チェック
            int disp_index = ui->stackedWidget->currentIndex();

            if(disp_index != DISP_scrTransfer )
            {
                //入力画面切り替え
                ui->stackedWidget->setCurrentIndex(DISP_scrTransfer);
                ui->tabWidget->setCurrentIndex(0);

                //初期値設定
                MainWindow::initPlotLabel();
                MainWindow::initPlotStyle();
            }


            //現在表示されているページ番号を設定
            MainWindow::m_out_page_no = ui->stackedWidget_2->currentIndex();

            //display_indexの種別を取得
            int index =  MainWindow::GetPlotTitle(int(DISP_TF) ,int(DISP_FREQ) , int(DISP_SRC) );

            //display_indexにデータ設定
            MainWindow::setDispIndex(index);

            break;
        }
        //---------------------------------------------------------
        // SPEC項目を選択した場合
        //---------------------------------------------------------
        case DISP_M2PG_DATA:       //data
        case DISP_M2PG_PLACE:      //Place
        case DISP_M2PG_ROBOT:      //Robot
        case DISP_M2PG_ARRAY_TYPE: //Array
        case DISP_M2PG_CREATOR:    //Creator
        case DISP_M2PG_NOTES:      //Note
            ui->m2pg_table->setEditTriggers(QAbstractItemView::AllEditTriggers);
            ui->m2pg_table->setStyleSheet("color: white;font: bold 16px;");
            break;

        default:
            break;
        }

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }


}






//-------------------------------------------------//
// plot Label 初期化
//-------------------------------------------------//
void MainWindow::initPlotLabel()
{
    //target
    ui->target->setCurrentIndex(int(DISP_TF));

    //x軸ラベル
    ui->x_combo->setCurrentIndex(int(DISP_FREQ));

    //Y軸ラベル
    ui->y_combo->setCurrentIndex(int(DISP_SRC));

    //Z軸ラベル
     ui->z_combo->setCurrentIndex(int(DISP_AMP));

     ui->x_min->setText("");
     ui->x_max->setText("");
     ui->y_min->setText("");
     ui->y_max->setText("");
     ui->z_min->setText("");
     ui->z_max->setText("");

}



//-------------------------------------------------//
// disp_indexのcomboBoxに値を設定
// in: disp_indexの値
//-------------------------------------------------//
void MainWindow::setDispIndex(int index)
{

    switch(index)
    {
    case DISP_MIC:
            MainWindow::setMicCombo();
            break;

    case DISP_SRC:
            break;

    case DISP_FREQ:
            break;

    }
}




//-------------------------------------------------//
//　plot style 初期化
//-------------------------------------------------//
void MainWindow::initPlotStyle()
{
    ui->plot_style->setCurrentIndex(DISP_LINES);
    ui->surface->setCurrentIndex(DISP_LINE);
    ui->color_map->setCurrentIndex(DISP_UNDEFINE);
}





//-----------------------------------------------//
//  undefineをlabelに設定
//-----------------------------------------------//
void MainWindow::setTextLabel(QString label , QString text )
{
    if(label == "label_disp_index")
    {
        //Labelを設定
        ui->label_disp_index->setStyleSheet(" color: white;font: bold 16px;");
        ui->label_disp_index->setText(text);
    }
}





//-----------------------------------------------//
//  Freq番号をcombo boxに設定
//-----------------------------------------------//
void MainWindow::setFreqCombo()
{
    try
    {
        //Labelを設定
        ui->label_disp_index->setStyleSheet(" color: white;font: bold 16px;");
        ui->label_disp_index->setText("Freq index for display");

        removeComboAll(ui->disp_index);

        int nfft = m_plotctl[m_out_page_no].param.NFFT; // FFT長
        int fs   = m_plotctl[m_out_page_no].param.NFS;   //サンプリング周波数

        for (int i = 0; i < nfft / 2; i++)
        {
            ui->disp_index->addItem(QString("%1(%2)").arg(i * fs / nfft).arg(i), QVariant(i));
        }

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }
}




//-----------------------------------------------//
// mic番号をcombo boxに設定
//-----------------------------------------------//
void MainWindow::setMicCombo()
{
    //すべて一旦削除
    removeComboAll(ui->disp_index);

    //新規作成
    int mic = m_plotctl[m_out_page_no].param.NMic;

    for (int i = 0; i < mic; i++)
    {
        ui->disp_index->addItem(QString("%1").arg(i), QVariant(i));
    }
}





//-----------------------------------------------//
// Timeをcombo boxに設定
//-----------------------------------------------//
void MainWindow::setTimeCombo()
{
    try
    {

        removeComboAll(ui->disp_index);

        int nfft = m_plotctl[m_out_page_no].param.NFFT; // FFT長
        int fs   = m_plotctl[m_out_page_no].param.NFS;   //サンプリング周波数

        for (int i = 0; i < nfft / 2; i++)
        {
            ui->disp_index->addItem(QString("%1(%2)").arg(double(i)/ fs, 7,'f',5).arg(i), QVariant(i));
        }

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }

}




//-----------------------------------------------//
// SRCをcombo boxに設定
//-----------------------------------------------//
void MainWindow::setSrcPosCombo()
{
    try
    {
        removeComboAll(ui->disp_index);

        //現在表示されているページのharkioのデータを格納
        int page_no = ui->stackedWidget_2->currentIndex();

        harkio::FMT *fmt = m_plotctl[page_no].harkio->getData()->getChunk<harkio::FMT>();

        int id = fmt->getFormatId();

        int src = 72;
        switch(id)
        {
        case harkio::FMT::M2PG:
            src =  m_plotctl[m_out_page_no].param.M2PG_ND;
            break;
        case harkio::FMT::GTF:
            src =  m_plotctl[m_out_page_no].param.GTF_NSRC;
            break;
        }

        for (int i = 0; i < src; i++)
        {
            ui->disp_index->addItem(QString("%1").arg(i), QVariant(i));
        }

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }
}





//-------------------------------------------//
// comboBox内の内容をすべて消す
//-------------------------------------------//
void MainWindow::removeComboAll(QComboBox *combo)
{
    while (1)
    {
        int index = combo->currentIndex();
        if (index < 0)
        {
            break;
        }
        combo->removeItem(index);
    }
}





//---------------------------------------------------//
// Graphボタン押下時処理
//---------------------------------------------------//
void MainWindow::on_Button_Graph_clicked()
{

    qDebug() << __func__  <<  " Graphボタン押下";

    //入力画面チェック
    int disp_index = ui->stackedWidget->currentIndex();

    int text_check = 0;
    switch(m_out_page_no)
    {
    //template時
    case DISP_srcIMPout:
    case DISP_srcTSPout:
    case DISP_srcMICout:
    case DISP_srcNOISEout:
        dispGraphTemplate();
        break;

    case DISP_srcM2PGout:
        text_check = ui->m2pg_table->item(3,1)->text().isEmpty();
        break;

    case DISP_scrGTFout:
        text_check = ui->gtf_table->item(3,1)->text().isEmpty();
        break;

    default:
        THROW_CRITICAL("Error OpuPut Display");
        return;
    }


    //データが記入されている場合のみ
    if(disp_index == DISP_scrTransfer && text_check == FALSE )
    {
        dispGraph();
    }

}





//---------------------------------------------------//
//  Z軸Label表示
//---------------------------------------------------//
std::string MainWindow::zUnitName(int id)
{

    std::string label;

    switch(id)
    {
    case DISP_AMP:
    case DISP_REAL:
        label = "Amplitude";
        break;
    case DISP_PHASE:
        label = "Phase[deg.]";
        break;
    case DISP_db:
        label = "[dB]";
        break;
    default:
        MainWindow::THROW_CRITICAL("Unknow Z axis's unit");
        break;
    }
    return label;
}






//---------------------------------------------------//
//  templateのグラフ作成処理
//---------------------------------------------------//
void  MainWindow::dispGraphTemplate()
{
    try
    {
        qDebug() << "dispGraphTemplate() start";

        switch(m_kind)
        {
        case RUN_IMP:
        {
            QString text = ui->imp_textEdit->toPlainText();
            std::vector<std::vector<double> > lists; // マイク位置

            // 一時ディレクトリを生成
            boost::filesystem::path temp_dir = CreateTemporaryDirectory();
            boost::filesystem::path hark_config_file_path = temp_dir / "ImpulseResponseListFile.xml";
            QFile hark_config(hark_config_file_path.c_str());

            qDebug() << "impulse tmp file = "<< hark_config_file_path.c_str();

            if (!hark_config.open(QIODevice::WriteOnly))
            {
                THROW_CRITICAL("can't open the temporary file");
                return;
            }

            QTextStream out(&hark_config);
            out << text << flush;

            boost::shared_ptr<harkio2::harkconfig::ImpulseResponseListFile2_0> ir_file =
                boost::dynamic_pointer_cast<harkio2::harkconfig::ImpulseResponseListFile2_0>(
                        harkio2::harkconfig::HarkConfigFileXMLReader(hark_config_file_path.c_str()).Read());

            if (!ir_file)
            {
               THROW_CRITICAL("invalid ImpulseResponseListFile");
               return;
            }

            std::vector<double> xyz(3);
            const auto& sources = ir_file->GetSources();
            for (size_t i = 0; i < sources.size(); ++i) {
                xyz[0] = sources[i].GetPosition().GetPositionVector().GetX();
                xyz[1] = sources[i].GetPosition().GetPositionVector().GetY();
                xyz[2] = sources[i].GetPosition().GetPositionVector().GetZ();
                lists.push_back(xyz);
            }

            // プロット
            ScatterPlot graph(&lists);
            graph.setTitle("Source Position");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());

            std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
            int ret = system(temp.c_str());
            qDebug() << __func__ << "system=" << ret;
            break;
        }


        case RUN_MIC:
        {
            QString text = ui->mic_textEdit->toPlainText();
            std::vector<std::vector<double> > lists; // マイク位置

            // 一時ディレクトリを生成
            boost::filesystem::path temp_dir = CreateTemporaryDirectory();
            boost::filesystem::path hark_config_file_path = temp_dir / "MicArrayLocationFile.xml";
            QFile hark_config(hark_config_file_path.c_str());

            if (!hark_config.open(QIODevice::WriteOnly))
            {
                THROW_CRITICAL("can't open the temporary file");
                return;
            }

            QTextStream out(&hark_config);
            out << text << flush;

            boost::shared_ptr<harkio2::harkconfig::MicArrayLocationFile2_0> mic_file =
                boost::dynamic_pointer_cast<harkio2::harkconfig::MicArrayLocationFile2_0>(
                        harkio2::harkconfig::HarkConfigFileXMLReader(hark_config_file_path.c_str()).Read());

            if (!mic_file) {
                THROW_CRITICAL("invalid MicArrayLocationFile");
                return;
            }

            std::vector<double> xyz(3);
            const auto& mics = mic_file->GetMics();
            for (size_t i = 0; i < mics.size(); ++i) {
                xyz[0] = mics[i].GetPosition().GetPositionVector().GetX();
                xyz[1] = mics[i].GetPosition().GetPositionVector().GetY();
                xyz[2] = mics[i].GetPosition().GetPositionVector().GetZ();
                lists.push_back(xyz);
            }


            // プロット
            ScatterPlot graph(&lists);
            graph.setTitle("MICARY-LocationFile");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());

            std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
            int ret = system(temp.c_str());
            qDebug() << __func__ << "system=" << ret;
            break;

        }
       case RUN_NOISE:
       {
            QString text = ui->noise_textEdit->toPlainText();
            std::vector<std::vector<double> > lists; // マイク位置

            // 一時ディレクトリを生成
            boost::filesystem::path temp_dir = CreateTemporaryDirectory();
            boost::filesystem::path hark_config_file_path = temp_dir / "NoiseLocationFile.xml";
            QFile hark_config(hark_config_file_path.c_str());

            if (!hark_config.open(QIODevice::WriteOnly))
            {
                THROW_CRITICAL("can't open the temporary file");
                return;
            }

            QTextStream out(&hark_config);
            out << text << flush;

            boost::shared_ptr<harkio2::harkconfig::NoiseLocationFile2_0> noise_file =
                boost::dynamic_pointer_cast<harkio2::harkconfig::NoiseLocationFile2_0>(
                        harkio2::harkconfig::HarkConfigFileXMLReader(hark_config_file_path.c_str()).Read());

            if (!noise_file) {
                THROW_CRITICAL("invalid NoiseLocationFile");
                return;
            }

            std::vector<double> xyz(3);
            const auto& noises = noise_file->GetNoiseSources();
            for (size_t i = 0; i < noises.size(); ++i) {
                xyz[0] = noises[i].GetPosition().GetPositionVector().GetX();
                xyz[1] = noises[i].GetPosition().GetPositionVector().GetY();
                xyz[2] = noises[i].GetPosition().GetPositionVector().GetZ();
                lists.push_back(xyz);
            }

            // プロット
            ScatterPlot graph(&lists);
            graph.setTitle("Noise-LocationFile");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());

            std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
            int ret = ::system(temp.c_str());
            qDebug() << __func__ << "system=" << ret;
            break;

        }

        case RUN_TSP:
        {
            QString text = ui->tsp_textEdit->toPlainText();
            std::vector<std::vector<double> > lists; // マイク位置

            // 一時ディレクトリを生成
            boost::filesystem::path temp_dir = CreateTemporaryDirectory();
            boost::filesystem::path hark_config_file_path = temp_dir / "TSPListFile.xml";
            QFile hark_config(hark_config_file_path.c_str());

            if (!hark_config.open(QIODevice::WriteOnly)) {
                THROW_CRITICAL("can't open the temporary file");
            }
            QTextStream out(&hark_config);
            out << text << flush;

            boost::shared_ptr<harkio2::harkconfig::TSPListFile2_0> tsp_file =
                boost::dynamic_pointer_cast<harkio2::harkconfig::TSPListFile2_0>(
                        harkio2::harkconfig::HarkConfigFileXMLReader(hark_config_file_path.c_str()).Read());

            if (!tsp_file)
            {
                THROW_CRITICAL("invalid TSPListFile");
                return;
            }

            std::vector<double> xyz(3);
            const auto& sources = tsp_file->GetSources();
            for (size_t i = 0; i < sources.size(); ++i) {
                xyz[0] = sources[i].GetPosition().GetPositionVector().GetX();
                xyz[1] = sources[i].GetPosition().GetPositionVector().GetY();
                xyz[2] = sources[i].GetPosition().GetPositionVector().GetZ();
                lists.push_back(xyz);
            }

            // プロット
            ScatterPlot graph(&lists);
            graph.setTitle("Source Position");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());

            std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
            int ret = system(temp.c_str());
            qDebug() << __func__ << "system=" << ret;
            break;

        }

        default:
            MainWindow::THROW_CRITICAL( QString("invalid m_kind=%1").arg(m_kind) );
            break;

        }//switch end

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }

}






//--------------------------------------------------------//
//  表示対象から分岐処理をする
//--------------------------------------------------------//
void  MainWindow::dispGraph()
{

    qDebug() <<__func__;

    int ret = 0;

    try
    {

        //現在表示されているページのharkioのデータを格納
        int page_no = ui->stackedWidget_2->currentIndex();

        harkio::FMT *fmt = m_plotctl[page_no].harkio->getData()->getChunk<harkio::FMT>();

        m_plotctl[page_no].param.NMic =  fmt->getNMic();  //mic数
        m_plotctl[page_no].param.NFFT =  fmt->getNFFT(); // FFT長
        m_plotctl[page_no].param.NIR  =  fmt->getNIR();  // インパルスレスポンス長
        m_plotctl[page_no].param.NFS  =  fmt->getNFS();  //サンプリング周波数


         // 表示対象取得
        int index  = ui->target->currentIndex();


        switch (index)
        {
        case DISP_TF:
        {

            ret = presetTFGraph(); //前処理

            if(ret == 0)
            {
                m_gnuplot.plot(MainWindow::harktoolDir());
            }

            break;
        }
        case DISP_IFFT:

            presetIFFTGraph();
            m_gnuplot.plot(MainWindow::harktoolDir());
            break;

//        case TARGET_DIFF:
//           {
//                GNUPlot graph;
//                presetDiffGraph(graph);
//                graph.plot(harktoolDir()); // グラフのプロット
//            }
//            break;

        default:
            THROW_CRITICAL("Unknown GraphID");
            break;
        }

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }
}









//--------------------------------------------------------//
//
//--------------------------------------------------------//
int MainWindow::otherUnit(int target, int x_unit, int y_unit)
{

    if (x_unit == y_unit)
    {
        return UNDEFINE;
    }

    if (target == DISP_TF || target == DISP_DIFF)
    {
        int total = AXIS_FREQ + AXIS_SRC + AXIS_MIC;
        return total - (x_unit + y_unit);
    }
    else if (target == DISP_IFFT)
    {
        int total = AXIS_TIME +  AXIS_SRC + AXIS_MIC;
        return total - (x_unit + y_unit);
    }
    else
    {
        MainWindow::THROW_CRITICAL( QString("Unknown unit id:%1").arg(target) );
    }

    return(ERROR);
}





//----------------------------------------------------
//display_indexの種別を取得
//----------------------------------------------------
int MainWindow::GetPlotTitle(int target, int x_unit, int y_unit)
{

    int total = x_unit + y_unit;

    switch(target)
    {
    case DISP_TF:
    {
        switch(total)
        {
        case 1://FREQ:0 + SRC:1 ->MIC
            return(DISP_MIC); //2
        case 2://FREQ:0 + MIC:2 ->SRC
            return(DISP_SRC); //1
        case 3://SRC:2  + MIC:1 ->FRQ
            return(DISP_FREQ);//0
        default:
            return(ERROR);//-1
        }
        break;
    }

    case DISP_IFFT:

        switch(total)
        {
        case 1://TIME:0 + SRC:1 ->MIC
            return(DISP_MIC); //2
        case 2://TIME:0 + MIC:2 ->SRC
            return(DISP_SRC); //1
        case 3://SRC:2  + MIC:1 ->TIME:0
            return(DISP_TIME);//0
        default:
            return(ERROR);//-1
        }
        break;

//    case TARGET_DIFF:
//    {
//        break;
//    }

    default:
        {
            MainWindow::THROW_CRITICAL(QString("Unknown unit id:%1").arg(target));
            break;
        }
    }

    return(ERROR);//-1

}





//----------------------------------------------------
// マトリックスの各次元は固定されているとみなして、GNUPlot用インデックスを作成する
//----------------------------------------------------
void MainWindow::setIndex(std::vector<int> *indexes, int xyz_axis, int specific_axis)
{
    switch (specific_axis)
    {
    case DISP_FREQ: // 0
//  case AXIS_TIME:
            (*indexes)[0] = xyz_axis;
        break;
    case DISP_MIC:  //2
        (*indexes)[1] = xyz_axis;
        break;
    case DISP_SRC: //1
        (*indexes)[2] = xyz_axis;
        break;
    default:
        THROW_CRITICAL("Unknown axis");
        break;
    }

}





//----------------------------------------------------
//
//----------------------------------------------------
void MainWindow::createTFTitle(std::string &title)
{

    std::ostringstream os;

    int x_axis = ui->x_combo->currentIndex();
    int y_axis = ui->y_combo->currentIndex();
    int axis   = MainWindow::GetPlotTitle(DISP_TF, x_axis,y_axis);
    int index  = ui->disp_index->currentIndex();

    int fs  = m_plotctl[m_out_page_no].param.NFS;  //サンプリング周波数
    int fft = m_plotctl[m_out_page_no].param.NFFT; //FFT長


    switch(axis)
    {
    case DISP_SRC:
        os << " Transfer Function of Src. No. " << index;
        break;
    case DISP_FREQ:
        os << " Transfer Function of " << index * fs / fft << " [Hz]";
        break;
    case DISP_MIC:
        os << " Transfer Function of Mic No. " << index;
        break;
    default:
        THROW_CRITICAL(QString("Unknown other parameter:%1").arg(axis));
        break;
    }

    title = os.str();

}





//--------------------------------------------------------//
// グラフのプロット前処理
//--------------------------------------------------------//
int MainWindow::presetTFGraph()
{

    try
    {
        qDebug() << __func__;

        int ret = 0;

        // 軸の定義
        std::vector<int> indexes(3);
        int x_axis;
        int y_axis;
        int z_axis;
        int z_axis_index;


        x_axis = ui->x_combo->currentIndex();//FREQ:0 SRC:1 MIC:2
        y_axis = ui->y_combo->currentIndex();//FREQ:0 SRC:1 MIC:2

        z_axis       = GetPlotTitle(DISP_TF, x_axis,y_axis); //disp_index値取得
        z_axis_index = ui->disp_index->currentIndex();       //disp_indexnの値


        // index 設定
        // FREQ,TIM->indexs[0] MIC->indexes[1],SRC->indexes[2]
        MainWindow::setIndex(&indexes, AXIS_X, x_axis);
        MainWindow::setIndex(&indexes, AXIS_Y, y_axis);
        MainWindow::setIndex(&indexes, z_axis_index, z_axis);


        // グラフオブジェクトの生成
        boost::multi_array<std::complex<double>, 3>* matrix;


        harkio::FMT *fmt   =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::FMT>();  //FMTチャンク取得

        //フォーマット種別取得(NDEFINE=0, M2PG, GTF, SM)
        int id = fmt->getFormatId();

        TFMatrix matrix_copy;

        switch(id)
        {
        case harkio::FMT::M2PG:
        {
            //M2PGチャンク取得
            harkio::M2PG *m2pg =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::M2PG>();
            harkio::M2PG::TF_LIST_TYPE *tf = m2pg->getTF();

            //HARKIO->Matrix変換(m2pg->getTF()) : HARKIOのデータをMatrix型に格納する
            m_transform.HARKIOTFToTFMatirxM2PG(tf , matrix_copy); //新規でMatrix配列を定義している
            break;
        }

        case harkio::FMT::GTF:
        {

              harkio::GTF *gtf =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::GTF>();
              harkio::GTF::TF_LIST_TYPE *tf = gtf->getTF();

              //HARKIO->Matrix変換(gtf->getTF()) : HARKIOのデータをMatrix型に格納する
              m_transform.HARKIOTFToTFMatirxGTF(tf , matrix_copy); //新規でMatrix配列を定義している
              break;
        }
        default:
            THROW_CRITICAL( QString("Unknown Format ID : %1").arg( __func__));
            return -1;
        }



        //boostのMatrixに変換
        matrix = matrix_copy.createBoostMatrix();

        ret = m_gnuplot.setMatrix(matrix, indexes);

        if(ret != 0)
        {
            return -1;
        }


        // グラフのタイトル
        std::string  title;
        MainWindow::createTFTitle(title);

        m_gnuplot.setTitle(title);


        // プロットスタイル
        int plot_style  = ui->plot_style->currentIndex();
        int surface     = ui->surface->currentIndex();
        int color_map   = ui->color_map->currentIndex();
        int z_unit      = ui->z_combo->currentIndex();

       m_gnuplot.setStyle(plot_style, surface, color_map);


        // Z軸の単位
       m_gnuplot.z_unit = z_unit;


        // 軸のラベル
        std::vector<std::string> labels(4);
        labels[0] = "f[Hz]";
        labels[1] = "Mic[No.]";
        labels[2] = "Src[No.]";
        labels[3] = zUnitName(z_unit);
        m_gnuplot.setLabels(labels);


        // 軸の重み
        int fs = fmt->getNFS(); //sampling Rate(16000)
        int nfft = fmt->getNFFT(); //FFT長(512)
        std::vector<double> weights(4);

        weights[0] = double(fs) / nfft; // FFT
        weights[1] = 1; // SRC
        weights[2] = 1; // MIC
        weights[3] = 1;
        m_gnuplot.setWeights(weights);


        // 軸の表示範囲
        std::string x_min = ui->x_min->text().toStdString();
        std::string x_max  = ui->x_max->text().toStdString();
        std::string y_min = ui->y_min->text().toStdString();
        std::string y_max  = ui->y_max->text().toStdString();
        std::string z_min = ui->z_min->text().toStdString();
        std::string z_max  = ui->z_max->text().toStdString();
        std::pair<std::string, std::string> x_range(x_min, x_max);
        std::pair<std::string, std::string> y_range(y_min, y_max);
        std::pair<std::string, std::string> z_range(z_min, z_max);
        m_gnuplot.setCurrentRange(x_range, y_range, z_range);

        return 0;

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
         return -1;
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
        return -1;
    }

}







//----------------------------------------------------
//  IFFTグラフの title
//----------------------------------------------------
void MainWindow::createIFFTTitle(std::string &title)
{
    try
    {
        std::ostringstream os;

        //現在表示されているページ番号
        MainWindow::m_out_page_no = ui->stackedWidget_2->currentIndex();


        int x_axis = ui->x_combo->currentIndex();
        int y_axis = ui->y_combo->currentIndex();
        int axis   = MainWindow::GetPlotTitle(DISP_IFFT, x_axis,y_axis);
        int index  = ui->disp_index->currentIndex();

        int fs  = m_plotctl[m_out_page_no].param.NFS;  //サンプリング周波数


        switch(axis)
        {
        case DISP_SRC:
            os << " IFFT of Src. No. " << index;
            break;
        case DISP_TIME:
            os << " IFFT of " << double(index) / fs << " [Sec]";
            break;
        case DISP_MIC:
            os << " IFFT of Mic No. " << index;
            break;
        default:
            THROW_CRITICAL(QString("Unknown other parameter:%1").arg(axis));
            break;
        }

        title = os.str();

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }

}







//--------------------------------------------------------//
// グラフのプロット前処理(IFFT)
//--------------------------------------------------------//
int MainWindow::presetIFFTGraph()
{
    try
    {
        qDebug() << __func__;

        int ret = 0;

       //現在表示されているページ番号取得
       m_out_page_no = ui->stackedWidget_2->currentIndex();


        // 軸の定義
        std::vector<int> indexes(3);
        int x_axis;
        int y_axis;
        int z_axis;
        int z_axis_index;


        x_axis = ui->x_combo->currentIndex();//FREQ:0 SRC:1 MIC:2
        y_axis = ui->y_combo->currentIndex();//FREQ:0 SRC:1 MIC:2

        z_axis       = GetPlotTitle(DISP_IFFT, x_axis,y_axis); //disp_index値取得
        z_axis_index = ui->disp_index->currentIndex(); //disp_indexnの値


        // index 設定
        // FREQ,TIM->indexs[0] MIC->indexes[1],SRC->indexes[2]
        MainWindow::setIndex(&indexes, AXIS_X, x_axis);
        MainWindow::setIndex(&indexes, AXIS_Y, y_axis);
        MainWindow::setIndex(&indexes, z_axis_index, z_axis);


        int NMic = m_plotctl[m_out_page_no].param.NMic;  //mic数
        int FFT = m_plotctl[m_out_page_no].param.NFFT; // FFT長
        int NSrc =0;


        // グラフオブジェクトの生成
        boost::multi_array<std::complex<double>, 3>* matrix;

         //FMTチャンク取得
        harkio::FMT *fmt   =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::FMT>();



        //フォーマット種別取得
        int id = fmt->getFormatId();

        TFMatrix matrix_copy;

        switch(id)
        {
        case harkio::FMT::M2PG:
        {
            //M2PGチャンク取得
            harkio::M2PG *m2pg =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::M2PG>();
            harkio::M2PG::TF_LIST_TYPE *tf = m2pg->getTF();
            NSrc =  m2pg->getND();//音源位置数

            //HARKIO->Matrix変換(m2pg->getTF()) : HARKIOのデータをMatrix型に格納する
            m_transform.HARKIOTFToTFMatirxM2PG(tf , matrix_copy); //新規でMatrix配列を定義している
            break;
        }

        case harkio::FMT::GTF:
        {
              harkio::GTF *gtf =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::GTF>();
              harkio::GTF::TF_LIST_TYPE *tf = gtf->getTF();
              NSrc = gtf->getNSrc(); //音源位置数

              //HARKIO->Matrix変換(gtf->getTF()) : HARKIOのデータをMatrix型に格納する
              m_transform.HARKIOTFToTFMatirxGTF(tf , matrix_copy); //新規でMatrix配列を定義している
              matrix_copy.conj();
              break;

        }
        default:
            THROW_CRITICAL( QString("format id unknown : id=%1 : %2").arg(id).arg( __func__));
            return -1;
        }


        TFMatrix &ifft_matrix=  MainWindow::IFFT( matrix_copy ,  FFT,  NMic,  NSrc);


        //boostのMatrixに変換
        matrix = ifft_matrix.createBoostMatrix();

        ret = m_gnuplot.setMatrix(matrix, indexes);

        if(ret != 0)
        {
            return -1;
        }


        // グラフのタイトル
        std::string  title;
        MainWindow::createIFFTTitle(title);

        m_gnuplot.setTitle(title);


        // プロットスタイル
        int plot_style  = ui->plot_style->currentIndex();
        int surface     = ui->surface->currentIndex();
        int color_map   = ui->color_map->currentIndex();
        int z_unit      = DISP_REAL;

       m_gnuplot.setStyle(plot_style, surface, color_map);


        // Z軸の単位
       m_gnuplot.z_unit = z_unit;


        // 軸のラベル
        std::vector<std::string> labels(4);
        labels[0] = "Time[Sec]";
        labels[1] = "Mic[No.]";
        labels[2] = "Src[No.]";
        labels[3] = zUnitName( z_unit );
        m_gnuplot.setLabels(labels);


        // 軸の重み
        int fs = fmt->getNFS(); //sampling Rate(16000)

        std::vector<double> weights(4);

        weights[0] = 1.0 / fs; // TIME
        weights[1] = 1; // SRC
        weights[2] = 1; // MIC
        weights[3] = 1;
        m_gnuplot.setWeights(weights);


        // 軸の表示範囲
        std::string x_min = ui->x_min->text().toStdString();
        std::string x_max  = ui->x_max->text().toStdString();
        std::string y_min = ui->y_min->text().toStdString();
        std::string y_max  = ui->y_max->text().toStdString();
        std::string z_min = ui->z_min->text().toStdString();
        std::string z_max  = ui->z_max->text().toStdString();
        std::pair<std::string, std::string> x_range(x_min, x_max);
        std::pair<std::string, std::string> y_range(y_min, y_max);
        std::pair<std::string, std::string> z_range(z_min, z_max);
        m_gnuplot.setCurrentRange(x_range, y_range, z_range);

        return 0;

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
         return -1;
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
        return -1;
    }
}






//--------------------------------------------------------//
//  IFFT
//--------------------------------------------------------//
TFMatrix&  MainWindow::IFFT( TFMatrix& matrix, int FFT, int NMic, int NSrc)
{

    try
    {

        // 結果を戻す配列
        TFMatrix *result_matrix = new TFMatrix(3, FFT, NMic, NSrc);


/*
#ifdef GPL

     {


        TFMatrix<HT_COMPLEX_DATA_TYPE> copy_matrix(DM3, FFT, NMic, NSrc); // 周波数ラインを元に戻す配列
        // matrix -> result へ(半分 + 1)コピー
        copy_matrix.range(DM3, FRQ, 0, FFT/2) = matrix.range(DM3, FRQ, 0, FFT/2);

        // 残り半分へ、複素共役をとってコピー
        for (int i=1; i < FFT/2; i++) { // 1から開始
    //		matrix.range(DM3, FRQ, i, i).conj();
            copy_matrix.range(DM3, FRQ, FFT-i, FFT-i) = matrix.range(DM3, FRQ, i, i);
            copy_matrix.range(DM3, FRQ, FFT-i, FFT-i).conj();
        }

        // 逆FFT
        fftw_complex *in, *out;
        fftw_plan p;

        in  = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * FFT);
        out = (fftw_complex *)fftw_malloc(sizeof(fftw_complex) * FFT);
        p   = fftw_create_plan(FFT, FFTW_BACKWARD, FFTW_ESTIMATE); 	// 逆FFT

        for(int mic =0; mic < NMic; mic++)
        {
            for(int src =0; src < NSrc; src++)
            {
                // fftw 領域へコピー
                for(int fft =0; fft < FFT; fft++)
                {
                    HT_COMPLEX_DATA_TYPE *mat = &(copy_matrix.matrix[fft][mic][src]);
                    in[fft].re = mat->real();
                    in[fft].im = mat->imag();
                }
                // Execute
                fftw_one(p, in, out);
                // 結果の取得
                for(int fft =0; fft < FFT; fft++) { // 同じループ
                    result_matrix->matrix[fft][mic][src] = HT_COMPLEX_DATA_TYPE(out[fft].re, out[fft].im);
                }
            }
        }
        fftw_destroy_plan(p);
        fftw_free(in);
        fftw_free(out);
    }

#endif
*/


//#ifdef HARKLICENSE
        qDebug("in HARKLICENCE");

        HRI::rfft<HT_DATA_TYPE> fft_obj(FFT);
        for (int mic = 0; mic < NMic; mic++)
        {
            for (int src = 0; src < NSrc; src++)
            {
                for (int fft = 0; fft < FFT/2.0+1; fft++) // fftwとの違いに注意
                {
                    fft_obj.A[fft] = matrix.matrix[fft][mic][src];
                }

                fft_obj.synthesize();
                for (int fft = 0; fft < FFT; fft++)
                {
                    result_matrix->matrix[fft][mic][src] = HT_COMPLEX_DATA_TYPE(fft_obj.a[fft], 0);
                }
            }
        }
//#endif

        return (*result_matrix);

    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("unexpected error"));
    }

}







//---------------------------------------------------//
// 終了ボタン押下時処理
//---------------------------------------------------//
void MainWindow::on_exit_Button_clicked()
{

    qDebug() << __func__;

    //---------------------------------------
    //  グラフ作成用テンポラリ削除
    //---------------------------------------
    QDir dir;
    QString path;

    path = dir.homePath();
    path = path + "/" + TEMPORARY_PATH.c_str() ;

    qDebug() << "path-->" << path ;


    if ( QFile::exists( path ) )
    {
        //存在する場合
        path = path + "/*" ;
        std::string temp = (boost::format("rm -rf %s") % path.toStdString() ).str();

        int ret = system(temp.c_str());

        if( WIFEXITED(ret) == -1)
        {
            printf("tmp file delete error:%d\n", WEXITSTATUS(ret));
        }

    }


    ui->title_label_2->setGeometry(10,10,1191,701);
    ui->title_label_2->setStyleSheet("background-color:black");
    ui->title_label_2->setFocus();
    ui->title_label_2->show();

    QTimer::singleShot( 1500, this, SLOT( close() ) );


}




//---------------------------------------------------//
// 削除処理
//---------------------------------------------------//
void MainWindow::tmp_dir_delete(QString strPath)
{

    if( strPath.isEmpty() != true )
    {
        QStringList list = strPath.split("/");
        QString tmp_dir = "/" + list.at(1) + "/" + list.at(2);
        std::cout << "tmp_dir=" << tmp_dir.toStdString() << std::endl;
    }
}






//---------------------------------------------------//
// 終了処理
//---------------------------------------------------//
void MainWindow::hark_close()
{
    qDebug() << __func__;
    close();
}








//-----------------------------------------------------------//
// Systemコマンドで起動したプロセスの終了処理
//  exitCode:終了する時に返したコード
//-----------------------------------------------------------//
void MainWindow::ProcFinished(int exitCode, QProcess::ExitStatus exitStatus)
{

    if ( exitStatus == QProcess::CrashExit || exitCode != 0)
    {
        QErrorMessage *w = new QErrorMessage;
        QString errmsg;

        switch(MainWindow::m_kind)
        {
        case RUN_MUSIC_CUI:
            errmsg =  "harktool4_cui error [Localization transfer function can not create]";
            break;

        case RUN_GTF_CUI:
            errmsg =  "harktool4_cui error [Separation transfer function can not create]";
            break;

        default:
            errmsg = QString("System command error[%1]").arg(m_kind);
        }

        w->showMessage(errmsg);


    }
    else
    {
        // 正常終了時の処理
        QString msg = QString("normal end run[%1] kind[%2] exitCode[%3]").arg(m_run).arg(m_kind).arg(exitCode);
        qDebug() << msg;

        switch(MainWindow::m_kind)
        {
         case RUN_MUSIC_CUI:

            qDebug("M2PGDataDisp() normal end");

            MainWindow::M2PGDataDisp();
            ui->progressBar->setValue(72);

            //M2PG画面にもどす
            setRunStatus(RUN_MUSIC);
            break;

        case RUN_GTF_CUI:
            qDebug("GTFDataDisp() end");

            MainWindow::GTFDataDisp();
            ui->progressBar->setValue(72);

            //GTF画面にもどす
            setRunStatus(RUN_GTF);
            break;

        default:
                break;
        }

    }

    qDebug("exitCode=%d, exitStatus=%d", exitCode, exitStatus);
//    MainWindow::m_run = STOP;  //停止に設定

}



//------------------------------------------------------//
// 実行中のプログレスバー表示処理
//------------------------------------------------------//
void MainWindow::ProcOutput()
{
    //標準出力
    QByteArray ab = m_Process.readAllStandardOutput();

    //    qDebug() << ab.data();
    std::cout << ab.data() << std::endl;

    if(m_bar < 1)
    {
      MainWindow::m_run = RUNNING; //実行中
      ui->check_label->setText("");
    }

    m_bar = m_bar + 1;

    if( m_bar > 72 )
    {
        m_bar = 0;
    }

    ui->progressBar->setValue(m_bar);

}





//------------------------------------------------------//
// //エラー
//------------------------------------------------------//
void MainWindow::ProcError()
{
    //標準エラー
    QByteArray ab = m_Process.readAllStandardError();
    qDebug() << ab.data();
}






//---------------------------------------------------//
//  color_map combox変更
//---------------------------------------------------//
void MainWindow::on_color_map_currentIndexChanged(int index)
{

    switch(index)
    {
    case DISP_PM3D:
        ui->plot_style->setCurrentIndex(DISP_UNDEFINE);
        ui->surface->setCurrentIndex(DISP_UNDEFINE);
//        ui->surface->setDisabled(true);
        break;

    case DISP_PM3D_MAP:
        ui->plot_style->setCurrentIndex(DISP_UNDEFINE);
        ui->surface->setCurrentIndex(DISP_UNDEFINE);
//        ui->plot_style->setDisabled(true);
//        ui->surface->setDisabled(true);
        break;

    case DISP_PM3D_AT_B:
        ui->plot_style->setCurrentIndex(DISP_UNDEFINE);
        ui->surface->setCurrentIndex(DISP_UNDEFINE);
//        ui->surface->setDisabled(true);
        break;
    }

}





//---------------------------------------------------//
// comboboxの値変更時(target=TF)
// input: new x_comboBox no
//      : flg=変更したcombobox
//---------------------------------------------------//
void MainWindow::ChangeComboBOX_TF(int x_index , int y_index)
{

    switch(x_index)
    {
    // 周波数
    case DISP_FREQ:

        switch(y_index)
        {
         case DISP_FREQ:
            setTextLabel("label_disp_index" ,"undefine"  );
            ui->disp_index->setDisabled(true);   //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(true); //Graphボタンをdisable設定
            break;

        case DISP_SRC:
            setTextLabel("label_disp_index" ,"Mic index for display"  );
            setMicCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;

        case DISP_MIC:
            setTextLabel("label_disp_index" ,"SRC index for display"  );
            setSrcPosCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;
        }
        break;



    // SRC
    case DISP_SRC:

        switch(y_index)
        {
         case DISP_FREQ:
            setTextLabel("label_disp_index" ,"Mic index for display"  );
            setMicCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;


        case DISP_SRC:
            setTextLabel("label_disp_index" ,"undefine"  );
            ui->disp_index->setDisabled(true); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(true);//Graphボタンをdisable設定
            break;


        case DISP_MIC:
            setTextLabel("label_disp_index" ,"Freq index for display"  );
            setFreqCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;
        }
        break;



    // MIC
    case DISP_MIC:

        switch(y_index)
        {
         case DISP_FREQ:
            setTextLabel("label_disp_index" ,"SRC index for display"  );
            setSrcPosCombo(); //srcを設定
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;


        case DISP_SRC:
            setTextLabel("label_disp_index" ,"Freq index for display"  );
            MainWindow::setFreqCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;


        case DISP_MIC:
            setTextLabel("label_disp_index" ,"undefine"  );
            ui->disp_index->setDisabled(true); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(true);//Graphボタンをdisable設定
            break;
        }
        break;
    }
}








//---------------------------------------------------//
// x軸,y軸　comboboxの値変更時(target=IFFT)
// input: flg= 変更した軸（x軸 or y軸）
//---------------------------------------------------//
void MainWindow::ChangeComboBOX_IFFT(int x_index , int y_index)

{

    switch(x_index)
    {
    case DISP_TIME:

        // y軸取得
        switch(y_index)
        {
         case DISP_TIME:
            setTextLabel("label_disp_index" ,"undefine"  );
            ui->disp_index->setDisabled(true); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(true);//Graphボタンをdisable設定
            break;

        case DISP_SRC:
            setTextLabel("label_disp_index" ,"Mic index for display"  );
            MainWindow::setMicCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;

        case DISP_MIC:
            setTextLabel("label_disp_index" ,"SRC index for display"  );
            MainWindow::setSrcPosCombo(); //srcを設定
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;
        }
        break;



    case DISP_SRC:

        switch(y_index)
        {
         case DISP_TIME:
            setTextLabel("label_disp_index" ,"Mic index for display"  );
            MainWindow::setMicCombo();
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;


        case DISP_SRC:
            setTextLabel("label_disp_index" ,"undefine"  );
            ui->disp_index->setDisabled(true); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(true);//Graphボタンをdisable設定
            break;


        case DISP_MIC:
            setTextLabel("label_disp_index" ,"TIME index for display"  );
            setTimeCombo();  //timeを設定
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;
        }
        break;


    case DISP_MIC:

        switch(y_index)
        {
         case DISP_TIME:
            setTextLabel("label_disp_index" ,"SRC index for display"  );
            MainWindow::setSrcPosCombo(); //srcを設定
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;


        case DISP_SRC:
            setTextLabel("label_disp_index" ,"TIME index for display"  );
            MainWindow::setTimeCombo();  //timeを設定
            ui->disp_index->setDisabled(false); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(false);//Graphボタンをdisable設定
            break;


        case DISP_MIC:
            setTextLabel("label_disp_index" ,"undefine"  );
            ui->disp_index->setDisabled(true); //disp_indexをdisable設定
            ui->Button_Graph->setDisabled(true);//Graphボタンをdisable設定
            break;
        }
        break;
    }
}









//---------------------------------------------------//
//  x軸　comboboxの値変更時の振り分け処理
//DISP_TIME =  0 ,DISP_UNDEFINE
//DISP_FREQ =  0 ,
//DISP_SRC  =  1 ,
//DISP_MIC  =  2
//#define DISP_TF      0
//#define DISP_IFFT    1
//#define DISP_DIFF    2

//---------------------------------------------------//
void MainWindow::on_x_combo_currentIndexChanged(int x_index)
{

   int y_index = ui->y_combo->currentIndex();

    //target combobox
    switch(ui->target->currentIndex())
    {
    case DISP_TF:

        MainWindow::ChangeComboBOX_TF(x_index ,y_index);
        break;

    case DISP_IFFT:
        MainWindow::ChangeComboBOX_IFFT(x_index , y_index);
        break;

//    case DISP_DIFF:
//        break;

    }
}



//---------------------------------------------------//
//  Y軸　comboboxの値変更時
//DISP_TIME =  0 ,
//DISP_FREQ =  0 ,
//DISP_SRC  =  1 ,
//DISP_MIC  =  2
//---------------------------------------------------//
void MainWindow::on_y_combo_currentIndexChanged(int y_index)
{

    int x_index = ui->x_combo->currentIndex();

    //target combobox
    switch(ui->target->currentIndex())
    {
    case DISP_TF:

        MainWindow::ChangeComboBOX_TF(x_index ,y_index);
        break;

    case DISP_IFFT:
        MainWindow::ChangeComboBOX_IFFT(x_index , y_index);
        break;

//    case DISP_DIFF:
//        break;

    }

}

















//---------------------------------------------------//
//  PLOT設定画面の軸項目のRESET押下時
//---------------------------------------------------//
void MainWindow::on_reset_label_clicked()
{
    initPlotLabel();//初期設定
    ui->Button_Graph->setDisabled(false);
}



//---------------------------------------------------//
//  PLOT設定画面の軸項目のRESET押下時
//---------------------------------------------------//
void MainWindow::on_reset_param_clicked()
{
    initPlotStyle();
    ui->Button_Graph->setDisabled(false);
}




//---------------------------------------------------//
// target comboboxが変更されたときの処理
//combboxの値
//#define DISP_TF      0
//#define DISP_IFFT    1
//#define DISP_DIFF    2
//---------------------------------------------------//
void MainWindow::on_target_currentIndexChanged(int index)
{


    QString strFreq;
    QString strSrc ;
    QString strMic ;
    QString strTime;

    if ( QLocale::Japanese != QLocale::system().language() )
    {
        strFreq ="Frequency";
        strSrc  ="Sound Source Number";
        strMic  ="Microphone Number";
        strTime ="TIME";

    }
    else
    {
        strFreq ="周波数";
        strSrc  ="音源番号";
        strMic  ="マイク番号";
        strTime ="時間";
    }




    switch(index)
    {
    case DISP_TF:
        removeComboAll(ui->x_combo);
        ui->x_combo->addItem(strFreq);
        ui->x_combo->addItem(strSrc);
        ui->x_combo->addItem(strMic);
        ui->x_combo->setCurrentIndex(0);

        removeComboAll(ui->y_combo);
        ui->y_combo->addItem(strFreq);
        ui->y_combo->addItem(strSrc);
        ui->y_combo->addItem(strMic);
        ui->y_combo->setCurrentIndex(1);
        break;


    case DISP_IFFT:
        removeComboAll(ui->x_combo);
        ui->x_combo->addItem(strTime.toUtf8().data());
        ui->x_combo->addItem(strSrc);
        ui->x_combo->addItem(strMic);
        ui->x_min->setText("");
        ui->x_max->setText("");
        ui->x_combo->setCurrentIndex(0);

        removeComboAll(ui->y_combo);
        ui->y_combo->addItem(strTime);
        ui->y_combo->addItem(strSrc);
        ui->y_combo->addItem(strMic);
        ui->y_min->setText("");
        ui->y_max->setText("");
        ui->y_combo->setCurrentIndex(1);


        MainWindow::setDispIndex(DISP_MIC);

        break;


//    case DISP_DIFF:
//    break;

    }



}






//---------------------------------------------------//
// separation の　MicArrayLocatin file 選択ボタン
//---------------------------------------------------//
void MainWindow::on_gtf_file_Button_clicked()
{
    QString fname;

    if(m_LastPath == "")
    {
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "MicArray Location File",
                                                ".",
                                                tr("XML Files(*.xml)")
                                            );

    }
    else
    {
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "MicArray Location File",
                                                m_LastPath,
                                                tr("XML Files(*.xml)")
                                            );
    }


    ui->gtf_mic_file->setText(fname);
    m_LastPath = fname; //最後のパス格納

    qDebug() << fname.toStdString().c_str();

}






//---------------------------------------------------//
// separation の　TSP/IR-ListFile 選択ボタン
//---------------------------------------------------//
void MainWindow::on_gtf_list_Button_clicked()
{

    QString fname;

    if(m_LastPath == "")
    {
        // Adds filter for XML files
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "TSP/IR-ListFile",
                                                ".",
                                                tr("XML Files(*.xml)")
                                            );
    }
    else
    {
        // Adds filter for XML files
        fname = QFileDialog::getOpenFileName(
                                                this,
                                                "TSP/IR-ListFile",
                                                m_LastPath,
                                                tr("XML Files(*.xml)")
                                            );
    }


    ui->gtf_list_file->setText(fname);
    m_LastPath = fname; //最後のパス格納

    qDebug() << fname.toStdString().c_str();

}











//---------------------------------------------//
// gtf_tableをダブルクリックした場合
//---------------------------------------------//
void MainWindow::on_gtf_table_cellDoubleClicked(int row, int column)
{



    qDebug() << "QTableWidget:" << "row=" << row << "column=" << column ;

    ui->gtf_table->setEditTriggers(QAbstractItemView::NoEditTriggers);

    //現在表示されているページ番号を設定
    MainWindow::m_out_page_no = ui->stackedWidget_2->currentIndex();

    int text_check = ui->gtf_table->item(3,1)->text().isEmpty();

    //データが表示されていない場合
    if( text_check == TRUE)
    {
        return;
    }





    std::vector<std::vector<double> > lists;

    //---------------------------------------------------------
    //fmt mic
    //row=7 column=0 1
    //---------------------------------------------------------
    switch(row)
    {
    case DISP_MIC_POSITION:
        {
            harkio::FMT *fmt =  m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::FMT> ();
            harkio::FMT::MIC_LIST_TYPE *mic = fmt->getMic();

            // MIC_LIST_TYPE から listへ変換する
            for (uint i = 0; i < (*mic).size(); i++)
            {
                std::vector<double> xyz(3);
                xyz[0] = (*mic)[i].x;
                xyz[1] = (*mic)[i].y;
                xyz[2] = (*mic)[i].z;
                lists.push_back(xyz);
            }


            // gnuplotで表示する
            ScatterPlot graph(&lists);
            graph.setTitle("Microphone Position");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());
            break;
        }
    case DISP_SOURCE_LOCATION:
        {


            harkio::GTF *gtf = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::GTF>();
            harkio::GTF::SRC_LIST_TYPE *src = gtf->getSrc();

            // SRC_LIST_TYPE から listへ変換する
            for ( unsigned int i = 0; i < (*src).size() ; i++)
            {
                    std::vector<double> xyz(3);
                    xyz[0] = (*src)[i].x;
                    xyz[1] = (*src)[i].y;
                    xyz[2] = (*src)[i].z;
                    lists.push_back(xyz);
             }

            // gnuplotで表示する
            ScatterPlot graph(&lists);
            graph.setTitle("Source Position");
            graph.setStyle(PlotStyle_POINTS);
            graph.plot(harktoolDir());
            break;
        }

    //---------------------------------------------------------
    //trance func のグラフ表示
    //row=12 column=0 1
    //---------------------------------------------------------
    case DISP_TRANSFER_FUNCTION:
    {


        //入力画面チェック
        int disp_index = ui->stackedWidget->currentIndex();



        if(disp_index != DISP_scrTransfer )
        {

            //入力画面切り替え
            ui->stackedWidget->setCurrentIndex(DISP_scrTransfer);
            ui->tabWidget->setCurrentIndex(0);

            //初期値設定
            MainWindow::initPlotLabel();

            MainWindow::initPlotStyle();
        }


        //display_indexの種別を取得
        int index =  MainWindow::GetPlotTitle(int(DISP_TF) ,int(DISP_FREQ) , int(DISP_SRC) );

        //display_indexにデータ設定
        MainWindow::setDispIndex(index);

        break;

    }

    case DISP_DATA: //data
    case DISP_PLACE: //Place
    case DISP_ROBOT: //Robot
    case DISP_ARRAY_TYPE: //Array
    case DISP_CREATOR: //Creator
    case DISP_NOTES: //Note
        ui->gtf_table->setEditTriggers(QAbstractItemView::AllEditTriggers);
        ui->gtf_table->setStyleSheet("color: white;font: bold 16px;");
//        ui->gtf_table->item(row,column)->setBackgroundColor(QColor("magenta"));

        break;

    default:
        break;
    }
}






//---------------------------------------------------//
// table のセルを変更した場合の処理
//---------------------------------------------------//
void MainWindow::on_gtf_table_cellChanged(int row, int column)
{
    qDebug() << __func__ << " row=" << row << " column=" << column;
    ui->gtf_table->setStyleSheet("color: white;font: bold 16px;");
}








//-------------------------------------------------//
//  open メニュー選択時の処理
//-------------------------------------------------//
void MainWindow::on_action_open_triggered()
{

    try
    {
        QFileInfo fileinfo = QFileInfo(m_LastPath);
        QString strFName  = QFileDialog::getOpenFileName(this, tr("Open File") , fileinfo.absolutePath() , tr("XML Files (*.xml);;ALL Files(*)" ) );

        if ( strFName.isEmpty() )
        {
            return;
        }

        std::ifstream ifs(strFName.toStdString().c_str());
        if(ifs.fail())
        {
            MainWindow::THROW_CRITICAL(QString(tr("Can't open file:%1")).arg(strFName));
        }

        //file read
        std::stringstream read_file;
        read_file << ifs.rdbuf();


        //file check
    //     file_name = strFName.toStdString();

        //ファイルタイプ確認(xmlファイルか？)
        auto src_file_type = GetHarkConfigFileType(strFName.toStdString());


        switch( src_file_type.second )
        {
        case HarkConfigFileType::ImpulseResponseListFile:
            ui->stackedWidget->setCurrentIndex(DISP_scrIMPtemplate);
            ui->stackedWidget_2->setCurrentIndex(DISP_srcIMPout);
            ui->imp_textEdit->setStyleSheet(" color: white;font: bold 16px;");
            ui->imp_textEdit->setPlainText( read_file.str().c_str() );
            setTopLabel(DISP_scrIMPtemplate);
//            m_filelist[DISP_scrIMPtemplate] << ifs.rdbuf();
            break;

        case HarkConfigFileType::TSPListFile:
            ui->stackedWidget->setCurrentIndex(DISP_scrTSPtemplate);
            ui->stackedWidget_2->setCurrentIndex(DISP_srcTSPout);
            ui->tsp_textEdit->setStyleSheet(" color: white;font: bold 16px;");
            ui->tsp_textEdit->setPlainText( read_file.str().c_str() );
            setTopLabel(DISP_scrTSPtemplate);
            break;


        case  HarkConfigFileType::MicArrayLocationFile:
            ui->stackedWidget->setCurrentIndex(DISP_scrMICtemplate);
            ui->stackedWidget_2->setCurrentIndex(DISP_srcMICout);
            ui->mic_textEdit->setStyleSheet(" color: white;font: bold 16px;");
            ui->mic_textEdit->setPlainText( read_file.str().c_str() );
            setTopLabel(DISP_scrMICtemplate);
            break;

        case HarkConfigFileType::NoiseLocationFile:
            ui->stackedWidget->setCurrentIndex(DISP_scrNOISEtemplate);
            ui->stackedWidget_2->setCurrentIndex(DISP_srcNOISEout);
            ui->noise_textEdit->setStyleSheet(" color: white;font: bold 16px;");
            ui->noise_textEdit->setPlainText( read_file.str().c_str() );
            setTopLabel(DISP_scrNOISEtemplate);
            break;

        default:

           harkio::HARKIO_RET  ret = harkio::HARK::checkIdentifier(strFName.toStdString());

            if (ret ==  harkio::HARKIO_TRUE)
            {
                harkio::HARK hark;
                hark.readFile(strFName.toStdString());


                if (hark.getData()->getChunk<harkio::M2PG> ())
                {

                     m_LastPath = ""; //LastPath clear
                     ui->top_label->setStyleSheet("font: bold  23px ; color:white");
                     ui->top_label->setText("Localization Transfer Function");

                     setRunStatus(RUN_MUSIC);//set status

                     boost::filesystem::path  tmp_path(strFName.toStdString().c_str() );
                     m_plotctl[m_out_page_no].hark_file_path = tmp_path;


                     //-------------------------------------------------------------------//
                     // harkio  領域確保
                     //-------------------------------------------------------------------//
                     if(m_plotctl[m_out_page_no].harkio != NULL)
                     {
                         qDebug() << __func__ << "delete harkio" ;

                         delete m_plotctl[m_out_page_no].harkio;
                         m_plotctl[m_out_page_no].harkio = NULL;
                     }

                     m_plotctl[m_out_page_no].harkio = new harkio::HARK;

                     MainWindow::M2PGDataDisp();


                }
                else if (hark.getData()->getChunk<harkio::GTF> ())
                {

                    m_LastPath = ""; //LastPath clear
                    ui->top_label->setStyleSheet("font: bold  23px ; color:white");
                    ui->top_label->setText("Separation Transfer Function");

                    setRunStatus(RUN_GTF);//set status

                    boost::filesystem::path  tmp_path(strFName.toStdString().c_str() );
                    m_plotctl[m_out_page_no].hark_file_path = tmp_path;


                    //-------------------------------------------------------------------//
                    // harkio  領域確保
                    //-------------------------------------------------------------------//
                    if(m_plotctl[m_out_page_no].harkio != NULL)
                    {
                        qDebug() << __func__ << "delete harkio" ;

                        delete m_plotctl[m_out_page_no].harkio;
                        m_plotctl[m_out_page_no].harkio = NULL;
                    }

                    m_plotctl[m_out_page_no].harkio = new harkio::HARK;

                    MainWindow::GTFDataDisp();


                }
                else
                {
                    MainWindow::THROW_CRITICAL(tr("Invalid file type"));
                }

                return;
            }
            else
            {
                MainWindow::THROW_CRITICAL(tr("Invalid file type : %1", "%1が不正なファイルタイプです").arg(strFName));
                return;
            }

        }//switch end


    }
    catch (std::exception& exp)
    {
//        DISP_DIALOG(exp);
        return;
    }




}







//---------------------------------------------------//
//  保存処理
//---------------------------------------------------//
void MainWindow::on_action_save_triggered()
{

    int index = ui->stackedWidget_2->currentIndex();

    QString fname= "";
    QString defExtension = ".xml";



    switch(m_kind)
    {
    case RUN_IMP :
    case RUN_TSP :
    case RUN_MIC :
    case RUN_NOISE :
        fname = QFileDialog::getSaveFileName(this, tr("Save File"), "." , tr("XML Files (*.xml);;ALL Files(*)" ));
        break;

    case RUN_MUSIC :
    case RUN_GTF   :
        // Adds additional filter for XML files
        fname = QFileDialog::getSaveFileName(
                                                this,
                                                tr("Save File"),
                                                "." ,
                                                tr("TF Files (*.tf);;ALL Files(*);; XML Files(*.xml)" )
                                            );
        break;
    }


qDebug() << "fname=" << fname;



    if(fname.isEmpty())
    {
        return;
    }
    else {
        // Sets default file extension
        QFileInfo info(fname);

        if (info.suffix().isEmpty()) {
            fname += defExtension;
        }
    }

    QFile file(fname); //インスタンス化

    //file open
    if(!file.open(QIODevice::WriteOnly))
    {
        return;
    }

    QTextStream out(&file);

    switch(index)
    {
    case DISP_srcIMPout:
        out << ui->imp_textEdit->toPlainText();
        out.flush();
        break;

    case DISP_srcTSPout:
        out << ui->tsp_textEdit->toPlainText();
        out.flush();
        break;

    case DISP_srcMICout:
        out << ui->mic_textEdit->toPlainText();
        out.flush();
        break;

    case DISP_srcNOISEout:
        out << ui->noise_textEdit->toPlainText();
        out.flush();
        break;

    case DISP_srcM2PGout:
    {
        //SPEC データ更新
        harkio::SPEC *spec = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::SPEC>();
        spec->setDate( ui->m2pg_table->item(DISP_M2PG_DATA ,1)->text().toStdString().c_str() );
        spec->setLocation(ui->m2pg_table->item(DISP_M2PG_PLACE ,1)->text().toStdString().c_str() );
        spec->setRobot(ui->m2pg_table->item(DISP_M2PG_ROBOT ,1)->text().toStdString().c_str() );
        spec->setArrayType(ui->m2pg_table->item(DISP_M2PG_ARRAY_TYPE ,1)->text().toStdString().c_str() );
        spec->setCreator(ui->m2pg_table->item(DISP_M2PG_CREATOR ,1)->text().toStdString().c_str() );
        spec->setMemo(ui->m2pg_table->item(DISP_M2PG_NOTES ,1)->text().toStdString().c_str() );

        // 書き込み
        m_plotctl[m_out_page_no].harkio->writeFile(fname.toStdString().c_str());

        break;
    }
    case DISP_scrTransfer:
        break;

    case DISP_scrGTFout:
        {
            //SPEC データ更新
            harkio::SPEC *spec = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::SPEC>();
            spec->setDate( ui->gtf_table->item(DISP_DATA ,1)->text().toStdString().c_str() );
            spec->setLocation(ui->gtf_table->item(DISP_PLACE ,1)->text().toStdString().c_str() );
            spec->setRobot(ui->gtf_table->item(DISP_ROBOT ,1)->text().toStdString().c_str() );
            spec->setArrayType(ui->gtf_table->item(DISP_ARRAY_TYPE ,1)->text().toStdString().c_str() );
            spec->setCreator(ui->gtf_table->item(DISP_CREATOR ,1)->text().toStdString().c_str() );
            spec->setMemo(ui->gtf_table->item(DISP_NOTES ,1)->text().toStdString().c_str() );

            // 書き込み
            m_plotctl[m_out_page_no].harkio->writeFile(fname.toStdString().c_str());

            break;
        }
    }//switch end

    THROW_INFO( tr("saved"));
    file.close();


    /*---------------------------------------------------*/
    // create zip file
    /*---------------------------------------------------*/
    qint64 pid;
    QString filename;
    QStringList   LIST   ;
    int i;
    int ret;

    i = fname.indexOf(".");

    if( i > 0)
    {
         filename =   QString(   fname.left(i).toStdString().c_str()  )  +  ".zip" ;
    }
    else
    {
        filename = QString(  fname.toStdString().c_str()  )  +  ".zip" ;
    }


    switch(index)
    {
    case DISP_srcM2PGout:
        {
            LIST  <<   "-l"  <<  QString( fname.toStdString().c_str() )   <<  "-o"   <<   filename ;

            ret = QProcess::startDetached("harktoolcli-conv-tf",   LIST  , QDir::currentPath(), &pid );

            qDebug() << "harktoolcli-conv-tf ret =  "  << ret  <<  LIST.at(0) <<  LIST.at(1)  << LIST.at(2)  << LIST.at(3)          ;

            if (!ret)
            {
                  THROW_CRITICAL(   QString(   "Command error: harktoolcli-conv-tf  " ) +    LIST.at(0) +  LIST.at(1)  +  LIST.at(2) +  LIST.at(3)    );
            }

            break;
        }
    case DISP_scrGTFout:
        {
            LIST  <<   "-s"  <<  QString( fname.toStdString().c_str() )   <<  "-o"   <<   filename ;

            ret = QProcess::startDetached("harktoolcli-conv-tf",   LIST  , QDir::currentPath(), &pid );

            qDebug() << "harktoolcli-conv-tf ret =  "  << ret  <<  LIST.at(0) <<  LIST.at(1)  << LIST.at(2)  << LIST.at(3)          ;

            if (!ret)
            {
                  THROW_CRITICAL(   QString(   "Command error: harktoolcli-conv-tf -s "  )  +    QString(   fname.toStdString().c_str()   )   );
            }

            break;
        }

    }//switch end
}






void MainWindow::on_action_8_triggered()
{


    m_helpdialog.exec();


}


//----------------------------------------//
// clear ボタン押下時の処理
//----------------------------------------//
void MainWindow::on_Button_Clear_clicked()
{

   setInit(m_kind);
}





//----------------------------------------//
// 初期化処理
//----------------------------------------//
void MainWindow::setInit(int kind)
{

    QDir dir; // 初期値の設定
    QString path;


    switch(kind)
    {
    case RUN_IMP:

        //初期表示
        ui->imp_spherical->setChecked(true);

        on_imp_spherical_clicked();

        ui->imp_radiusfrom->setValue(1.00);
        ui->imp_elvfrom->setValue(16.70);
        ui->imp_elvto->setValue(16.70);
        ui->imp_elvint->setValue(0);
        ui->imp_heightfrom->setValue(0);
        ui->imp_heightto->setValue(0.01);
        ui->imp_heightint->setValue(0.01);
        ui->imp_azifrom->setValue(180.00);
        ui->imp_azito->setValue(175.00);
        ui->imp_aziint->setValue(5.00);
        ui->imp_cut_start->setValue(1);
        ui->imp_cut_end->setValue(0);
        ui->imp_mic_list->setText("0,1,2,3,4,5,6,7");
        ui->imp_mic_flg->setCheckState(Qt::Unchecked);


        // インパルス応答リストファイル
        path = QString("%1/D{AZIMUTH:%03d}_E{ELEVATION:%03d}_R100_ch{CHANNEL:%02d}.flt").arg(dir.homePath());
        ui->imp_filename->setText(path);

        ui->imp_textEdit->setText("");


        break;



    case RUN_TSP:
    {
        ui->tsp_spherical->setChecked(true);

        on_tsp_spherical_clicked();

        ui->tsp_radiusfrom->setValue(1.00);
        ui->tsp_elvfrom->setValue(16.70);
        ui->tsp_elvto->setValue(16.70);
        ui->tsp_elvint->setValue(0);
        ui->tsp_heightfrom->setValue(0);
        ui->tsp_heightto->setValue(0.01);
        ui->tsp_heightint->setValue(0.01);
        ui->tsp_azifrom->setValue(180.00);
        ui->tsp_azito->setValue(175.00);
        ui->tsp_aziint->setValue(5.00);
        ui->tsp_cut_start->setValue(1);
        ui->tsp_cut_end->setValue(0);


        ui->tsp_sync->setValue(16);
        ui->tsp_offset->setValue(16384);
        ui->tsp_signal_max->setValue(32768);
        ui->tsp_mic_list->setText("0,1,2,3,4,5,6,7");
        ui->tsp_mic_flg->setCheckState(Qt::Unchecked);


        //TSPリストファイル
        path = QString("%1/D{AZIMUTH:%03d}_E{ELEVATION:%03d}_R100.wav").arg(dir.homePath());
        ui->tsp_filename->setText(path);


        // オリジナルインパルス
        QString app_dir = QCoreApplication::applicationDirPath();
        path = QString("%1/%2/%3").arg(app_dir).arg(HARK_TOOLS_DIR.c_str()).arg(IMPULSE_ORG.c_str());
        ui->tsp_org_impfile->setText(path);

        ui->tsp_textEdit->setText("");

        break;

    }
    case RUN_MIC:
    {
        //初期表示
        ui->mic_spherical->setChecked(true);

        on_mic_spherical_clicked();

        ui->mic_radiusfrom->setValue(1.00);
        ui->mic_elvfrom->setValue(16.70);
        ui->mic_elvto->setValue(16.70);
        ui->mic_elvint->setValue(0);

        ui->mic_heightfrom->setValue(0);
        ui->mic_heightto->setValue(0.01);
        ui->mic_heightint->setValue(0.01);

        ui->mic_azifrom->setValue(180.00);
        ui->mic_azito->setValue(175.00);
        ui->mic_aziint->setValue(45.00);

        ui->mic_textEdit->setText("");
        ui->mic_mic_flg->setCheckState(Qt::Unchecked);
        ui->mic_mic_list->setText("0,1,2,3,4,5,6,7");

        break;
    }
    case RUN_NOISE:

        ui->nois_radius->setValue(1.00);
        ui->nois_elevation->setValue(270.00);
        ui->nois_azimuth->setValue(0);
        ui->noise_textEdit->setText("");
        break;

    case RUN_MUSIC:

        ui->stackedWidget->setCurrentIndex(DISP_scrM2PG);
        ui->stackedWidget_2->setCurrentIndex(DISP_srcM2PGout);

        path = QString("%1/YOUR_PATH/MICARY-LocationFile").arg(dir.homePath());
        ui->m2pg_mic_file->setText(path);

        path = QString("%1/YOUR_PATH/ImpulseResponse-ListFile or TSP-ListFile").arg(dir.homePath());
        ui->m2pg_list_file->setText(path);

        ui->m2pg_sampling_freq->setValue(16000);
        ui->m2pg_fft_length->setValue(512);
        ui->m2pg_tsp_length->setValue(16384);
        ui->m2pg_direct_length->setValue(32);

        M2PGDispClear(); //harkio表示画面をclear
        break;




    case RUN_GTF:

        ui->stackedWidget->setCurrentIndex(DISP_scrGTF);
        ui->stackedWidget_2->setCurrentIndex(DISP_scrGTFout);

        path = QString("%1/YOUR_PATH/MICARY-LocationFile").arg(dir.homePath());
        ui->gtf_mic_file->setText(path);

        path = QString("%1/YOUR_PATH/ImpulseResponse-ListFile or TSP-ListFile").arg(dir.homePath());
        ui->gtf_list_file->setText(path);

        ui->gtf_sampling_freq->setValue(16000);
        ui->gtf_fft_length->setValue(512);
        ui->gtf_tsp_length->setValue(16384);

        MainWindow::GTFDispClear();
        break;

    }
}














//----------------------------------------//
// Run status &  pageを設定
//----------------------------------------//
void MainWindow::setRunStatus(int proc)
{

    switch(proc)
    {
    case RUN_IMP:
        m_run  = START;
        m_kind = RUN_IMP;
        m_in_page_no  = DISP_scrIMPtemplate ;
        m_out_page_no = DISP_srcIMPout;

        ui->stackedWidget->setCurrentIndex(DISP_scrIMPtemplate);
        ui->stackedWidget_2->setCurrentIndex(DISP_srcIMPout);

        break;

    case RUN_TSP:
        m_run  = START;
        m_kind = RUN_TSP;
        m_in_page_no  = DISP_scrTSPtemplate ;
        m_out_page_no = DISP_srcTSPout;

        ui->stackedWidget->setCurrentIndex(DISP_scrTSPtemplate);
        ui->stackedWidget_2->setCurrentIndex(DISP_srcTSPout);
        break;

    case RUN_MIC:
        m_run  = START;
        m_kind = RUN_MIC;
        m_in_page_no  = DISP_scrMICtemplate ;
        m_out_page_no = DISP_srcMICout;

        ui->stackedWidget->setCurrentIndex(DISP_scrMICtemplate);
        ui->stackedWidget_2->setCurrentIndex(DISP_srcMICout);
        break;

    case RUN_NOISE:
        m_run  = START;
        m_kind = RUN_NOISE;
        m_in_page_no  = DISP_scrNOISEtemplate ;
        m_out_page_no = DISP_srcNOISEout;

        ui->stackedWidget->setCurrentIndex(DISP_scrNOISEtemplate);
        ui->stackedWidget_2->setCurrentIndex(DISP_srcNOISEout);
        break;

    case RUN_MUSIC:
        m_run  = START;
        m_kind = RUN_MUSIC;
        m_in_page_no  = DISP_scrM2PG ;
        m_out_page_no = DISP_srcM2PGout;

        ui->stackedWidget->setCurrentIndex(DISP_scrM2PG);
        ui->stackedWidget_2->setCurrentIndex(DISP_srcM2PGout);
        break;

    case RUN_GTF:
        m_run  = START;
        m_kind = RUN_GTF;
        m_in_page_no  = DISP_scrGTF ;
        m_out_page_no = DISP_scrGTFout;

        ui->stackedWidget->setCurrentIndex(DISP_scrGTF);
        ui->stackedWidget_2->setCurrentIndex(DISP_scrGTFout);
        break;

    case RUN_MUSIC_CUI:
        m_run  = START;
        m_kind = RUN_MUSIC_CUI;
        break;

    case RUN_GTF_CUI:
        m_run  = START;
        m_kind = RUN_GTF_CUI;
        break;

    default:
        MainWindow::THROW_CRITICAL(QString("%1 : %2%3").arg(__func__).arg("error proc=").arg(proc));
    }

    qDebug() << __func__ << ": m_in_page_no=" << m_in_page_no << "m_out_page_no=" << m_out_page_no;
}





//----------------------------------------//
//  pageを設定
//----------------------------------------//
void MainWindow::setPageNo(int proc)
{

    switch(proc)
    {
    case RUN_IMP:
        m_in_page_no  = DISP_scrIMPtemplate ;
        m_out_page_no = DISP_srcIMPout;
        break;

    case RUN_TSP:
        m_in_page_no  = DISP_scrTSPtemplate ;
        m_out_page_no = DISP_srcTSPout;
        break;

    case RUN_MIC:
        m_in_page_no  = DISP_scrMICtemplate ;
        m_out_page_no = DISP_srcMICout;
        break;

    case RUN_NOISE:
        m_in_page_no  = DISP_scrNOISEtemplate ;
        m_out_page_no = DISP_srcNOISEout;
        break;

    case RUN_MUSIC:
        m_in_page_no  = DISP_scrM2PG ;
        m_out_page_no = DISP_srcM2PGout;
        break;

    case RUN_GTF:
        m_in_page_no  = DISP_scrGTF ;
        m_out_page_no = DISP_scrGTFout;
        break;

    case RUN_MUSIC_CUI:
    case RUN_GTF_CUI:
        //プログレスバー表示
        m_in_page_no  = DISP_scrProgressBar ;
        break;

    default:
        QString msg = QString("%1 : %2%3").arg(__func__).arg("error proc=").arg(proc);
        MainWindow::THROW_CRITICAL(msg);
    }
}





//---------------------------------------------------//
// top label設定
//---------------------------------------------------//
void MainWindow::setTopLabel(int kind)
{

    ui->top_label->setStyleSheet("font: bold  23px ; color:white");

    switch(kind)
    {
    case DISP_scrIMPtemplate:
        ui->top_label->setText("ImpulusResponse List File Template Create");
        break;

    case DISP_scrTSPtemplate:
        ui->top_label->setText("TSP List File Template Create");
        break;

    case DISP_scrMICtemplate:
        ui->top_label->setText("MICARRAY Location File Template Create");
        break;

    case DISP_scrNOISEtemplate:
        ui->top_label->setText("NOISE List File Template Create");
        break;

    case DISP_scrM2PG:
        ui->top_label->setText("Separation Transfer Function");
        break;

    case DISP_scrGTF :
        ui->top_label->setText("ImpulusResponse List File Template Create");
        break;

    default:
        ui->top_label->setText("");
        break;

    }

}



//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::ButtonClearFocus()
{

    ui->Button_Clear->clearFocus();
    ui->button_Create->clearFocus();
    ui->Button_IMP->clearFocus();
    ui->button_Localization->clearFocus();
    ui->Button_MIC->clearFocus();
    ui->Button_NOISE->clearFocus();
    ui->Button_save2->clearFocus();
    ui->button_Separation->clearFocus();
    ui->Button_TSP->clearFocus();
    ui->Button_Graph->clearFocus();

}







//--------------------------------------------------------//
// elevation(仰角)数取得
//--------------------------------------------------------//
int MainWindow::CalcNumelv(   double elvfrom  // 仰角開始
                            , double elvto    // 仰角間隔
                            , double elvint)
{

    int numelv        = 0; // 仰角数
    double tcount     = 0;


    if( elvto != elvfrom)
    {
        tcount = (elvto - elvfrom) / elvint;
    }

    numelv = ( (int)floor(tcount) ) +1; //切りすて  // 仰角数

    qDebug("elvfrom[%f] elvint[%f] numelv[%d]\n", elvfrom,  elvint , numelv);

    return numelv;
}




//--------------------------------------------------------//
// height(高さ)
//--------------------------------------------------------//
int MainWindow::CalcNumheight(  double heightfrom // 高さ開始
                              , double heightto   // 高さ終了
                              , double heightint) // 高さ間隔
{

    int numheight = 0 ; // 高さ数
    double hcount = 0 ;


    if(heightto != heightfrom)
    {
        hcount = ((heightto*1000) - (heightfrom*1000)) / (heightint*1000);
    }

     numheight = ((int)floor(hcount)) + 1;//切りすて // 高さ数

    qDebug("heightfrom[%f] heightto[%f] heightint[%f] numheight[%d]\n", heightfrom, heightto, heightint , numheight);

    return numheight;

}




//--------------------------------------------------------//
// AZIMUTH(方位角)
//--------------------------------------------------------//
int MainWindow::CalcNumazi(   double azifrom
                            , double azito
                            , double aziint)

{

    double azifrom_tmp = azifrom;

    if(azifrom >= azito)
    {
        azifrom_tmp = azifrom - 360;
    }

    double fcount = double(azito - azifrom_tmp) / aziint;

    int numazi = (int)(floor(fcount)) + 1; // 方位角数(切り捨て)

    qDebug("azifrom[%f] azito[%f] aziint[%f] numazi[%d]\n", azifrom, azito, aziint , numazi);

    return numazi;


}











//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);

    switch (e->type())
    {

    case QEvent::LanguageChange:
//        ui->retranslateUi(this);
        break;
    case QEvent::PaletteChange:
        break;

case QEvent::MouseMove:
//        ui->pushButton->show();
//        ui->pushButton->setFocus();
        break;

    case QEvent::ActivationChange:

    default:
        break;
    }
}








//--------------------------------------------//
// check Box 1 OFF に設定
//--------------------------------------------//
void MainWindow::on_checkBox_1_clicked()
{
    if( ui->checkBox_1->isChecked() == TRUE )
    {
        ui->checkBox_2->setChecked(FALSE);
        ui->checkBox_3->setChecked(FALSE);
        ui->checkBox_4->setChecked(FALSE);
    }

}


//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::on_checkBox_2_clicked()
{
    if( ui->checkBox_2->isChecked() == TRUE )
    {
       ui->checkBox_1->setChecked(FALSE);
   }

}


//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::on_checkBox_3_clicked()
{
    if( ui->checkBox_3->isChecked() == TRUE )
    {
       ui->checkBox_1->setChecked(FALSE);
   }

}


//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::on_checkBox_4_clicked()
{
    if( ui->checkBox_4->isChecked() == TRUE )
    {
       ui->checkBox_1->setChecked(FALSE);
   }
}






//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::on_m2pg_table_cellPressed(int row, int column)
{
    qDebug() << "on_m2pg_table_cellPressed" << "row=" << row << " column=" << column;
    ui->m2pg_table->setEditTriggers(QAbstractItemView::NoEditTriggers);
}





//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::on_gtf_table_cellPressed(int row, int column)
{
    qDebug() << "row =" << row << " : column=" << column;
    ui->gtf_table->setEditTriggers(QAbstractItemView::NoEditTriggers);
}





//--------------------------------------------------------//
//
//--------------------------------------------------------//
void MainWindow::on_action_Exit_triggered()
{

    MainWindow::on_exit_Button_clicked();

}













//--------------------------------------------------------//
// XML->TEXT変換(TSP LIST FILE)
//--------------------------------------------------------//
void MainWindow::TSPconvXmlToText()
{

    try
    {

        QString text = ui->tsp_textEdit->toPlainText();


        // 一時ディレクトリを生成
        m_convfile = "";
        boost::filesystem::path temp_dir = CreateTemporaryDirectory();
        m_convfile = temp_dir / "convfile_path" ;
        QFile hark_config( m_convfile.c_str());

        if (!hark_config.open(QIODevice::WriteOnly)) {
            THROW_CRITICAL("can't open the temporary file");
        }

        QTextStream out(&hark_config);

        out << text << flush;

        hark_config.close();


        boost::shared_ptr<harkio2::harkconfig::TSPListFile2_0> tsp_file =
            boost::dynamic_pointer_cast<harkio2::harkconfig::TSPListFile2_0>(
                    harkio2::harkconfig::HarkConfigFileXMLReader( m_convfile.c_str()).Read());

        if (!tsp_file)
        {
            THROW_CRITICAL("invalid TSPListFile");
            return;
        }



        //画面表示
        ui->stackedWidget->setCurrentIndex(DISP_scrWork);


        QString outbuf   = "# TSP-ListFile\n";
//        outbuf  = outbuf + "# DateTime: \n";
//        outbuf  = outbuf + "TSP-ListFile\n";
//        outbuf  = outbuf + "Version 1.1\n";

        outbuf  = outbuf + "SynchronousAverage = "
                         +   ( QString("%1").arg( tsp_file->GetSynchronousAverage() ) ).toStdString().c_str()
                         +  " #Num of Synchronous averaging.\n";

        outbuf  = outbuf + "OriginalImpulseFltFile = "
                         +   ( tsp_file->GetOriginalImpulseFile().GetName() ).c_str()
                         + "\n";

        outbuf  = outbuf + "TSPOffset = "
                         +  (QString("%1").arg(tsp_file->GetTSPOffset()) ).toStdString().c_str()
                         + " # Starting with 1\n" ;

        outbuf  = outbuf + "SignalMax ="
                         +  ( QString("%1").arg(tsp_file->GetSignalMax()) ).toStdString().c_str()
                         + "\n";


        //mic channel
        const auto& sources = tsp_file->GetSources();


        //channel list
        QStringList list = text.split("\n");
        QString strCh;
        QString channel_list = "";


        for(int i=0 ; i< list.count() ; i++)
        {
            strCh =  list.at(i).trimmed();

            if( strCh.startsWith("</MicIndices>") )
            {
                break;
            }
            else if( strCh.startsWith("<MicIndex>") )
            {
                strCh.replace("<MicIndex>"  , " ");
                strCh.replace("</MicIndex>" , " ");
                channel_list =  channel_list + ",";
                channel_list =  channel_list + strCh.trimmed();
            }

        }//for end


        //最初のカンマ削除
        channel_list.replace( 0 , 1 , "");

        outbuf = outbuf + "ChannelList = " + channel_list +  " # Wav channel No. for use\n" ;
        outbuf = outbuf + "\n";
        outbuf = outbuf + "# List of Sound source position and TSP file.\n";
        outbuf = outbuf + "# x, y, z , TSP file name, IR start with potision in sample(starting with 1).\n";

        QString StrPos;

        for (size_t i = 0; i < sources.size(); ++i)
        {
            StrPos = QString("%1, %2%3 , %4%5 , %6%7, %8%9, %10%11").arg( sources[i].GetPosition().GetPositionVector().GetX() ,0,'f',4 )
                                                                  .arg("  ")
                                                                  .arg( sources[i].GetPosition().GetPositionVector().GetY() ,0,'f',4 )
                                                                  .arg("  ")
                                                                  .arg( sources[i].GetPosition().GetPositionVector().GetZ() ,0,'f',4 )
                                                                  .arg("  ")
                                                                  .arg( QString::fromStdString( ( boost::format("%s") % tsp_file->GetSources()[i].GetFiles()[0].GetName() ).str()))
                                                                  .arg("  ")
                                                                  .arg(tsp_file->GetSources()[i].GetCutStart() )
                                                                  .arg("  ")
                                                                  .arg(tsp_file->GetSources()[i].GetCutEnd());

            outbuf = outbuf + StrPos + "\n";
        }

        //表示
        ui->work_textEdit->setStyleSheet(" color: white;font: bold 16px;");
        ui->work_textEdit->setText(outbuf);

         //tmp delete
         std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
         int ret = system(temp.c_str());
         qDebug() << __func__ << " system=" << ret << " temp=" << temp.c_str() ;


    }
    catch(const harkio2::AnalyzeError& e)
    {
        THROW_CRITICAL(trUtf8("Can not read xml ."));
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("Can not change text from xml."));

    }

}







//--------------------------------------------------------//
//TSP LIST FILE TEXT->XML
//--------------------------------------------------------//
void MainWindow::TSPconvTextToXml()
{

    try
    {

        // 一時ディレクトリを生成
        m_convfile = "";
        boost::filesystem::path temp_dir = CreateTemporaryDirectory();
        m_convfile = temp_dir/"convfile_path";

        std::ofstream out2( m_convfile.c_str() );
        boost::shared_ptr<harkio2::harkconfig::HarkConfigFileXMLWriter> writer(new harkio2::harkconfig::HarkConfigFileXMLWriter(out2));
        boost::shared_ptr<harkio2::harkconfig::TSPListFile2_0> tsp20(new harkio2::harkconfig::TSPListFile2_0());


        QString work_text = ui->work_textEdit->toPlainText();
        QStringList list = work_text.split("\n");

        QString ChannelList ="";
        QStringList strList;
        QStringList chList ;
        vector<unsigned int> channelVector;
        int value;
        bool ok;

        for(int i=0;i< list.count();i++)
        {
            if( list.at(i).indexOf("wav") > 0 )
            {
                break;
            }

            if( list.at(i).startsWith("SynchronousAverage") )
            {
                strList.clear();
                strList =  list.at(i).split("=" );
                strList = strList.at(1).split("#");
                value = strList.at(0).toInt(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("SynchronousAverage is not a numeric."));

                    return;
                }

                tsp20->SetSynchronousAverage( value );
            }
            else if( list.at(i).startsWith("OriginalImpulseFltFile") )
            {
                strList.clear();
                strList =  list.at(i).split("=" );
                strList = strList.at(1).split("#");
                tsp20->SetOriginalImpulseFile(harkio2::TypedFilename( strList.at(0).toStdString()
                                                                      , harkio2::FileType::FLOAT));
            }
            else if( list.at(i).startsWith("TSPOffset") )
            {
                strList.clear();
                strList =  list.at(i).split("=" );
                strList = strList.at(1).split("#");
                value = strList.at(0).toInt(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("TSPOffset is not a numeric."));
                    return;
                }

                tsp20->SetTSPOffset( value );
                tsp20->SetTSPLength( value );
            }
            else if( list.at(i).startsWith("SignalMax") )
            {
                strList.clear();
                strList =  list.at(i).split("=" );
                strList = strList.at(1).split("#");
                value = strList.at(0).toInt(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("SignalMax is not a numeric."));
                    return;
                }

                tsp20->SetSignalMax( value );
            }
            else if( list.at(i).startsWith("ChannelList") )
            {
                strList.clear();
                strList =  list.at(i).split("=" );
                strList = strList.at(1).split("#");
                ChannelList = strList.at(0).trimmed();
                chList = ChannelList.split(",");

                //channel->vectorに設定
                for(int i=0;i<chList.size();i++)
                {
                    value = chList.at(i).toInt(&ok);
                    if (ok == 0 )
                    {
                        THROW_CRITICAL(trUtf8("ChannelList is not a numeric."));
                        return;
                    }

                    channelVector.push_back(value);
                }
            }

        }//for end


        const unsigned int nmic = chList.size();
        const harkio2::FileType::Kind sourceFileType = harkio2::FileType::FLOAT;
        const Glib::ustring comment = u8R"(created by harktool4)";

        tsp20->SetNumMic(nmic);
        tsp20->SetSourceFileType(sourceFileType);
        tsp20->SetComment(comment);

        //Sources sizeをカウント
        int  SourcesSize = 0;
        for(int i = 0 ;i< list.count();i++)
        {
            if( list.at(i).indexOf("wav") > 0 )
            {
                 SourcesSize++;
            }
        }


        std::vector<harkio2::harkconfig::Source2> sources1(SourcesSize);
        std::vector<harkio2::harkconfig::File1> files;


        //position部分を探す
        int cnt=0;
        for(int i = 0 ;i< list.count();i++)
        {
            if( list.at(i).indexOf("wav") > 0 )
            {
                QString str = list.at(i);
//                int idx = str.indexOf("%");
//                str = str.replace( idx ,2 , "," ); //%%をカンマに変換

                QStringList PosList = str.split(",");


                if(PosList.count() != 6 )
                {
                    THROW_CRITICAL( QString("Data is not enough.\n[%1]").arg( list.at(i).toStdString().c_str() ) ) ;
                    return;
                }

                //numeric check

                 float X= PosList.at(0).toFloat(&ok);
                 if (ok == 0 )
                 {
                     THROW_CRITICAL(trUtf8("X position is not a numeric."));
                     return;
                 }

                 float Y= PosList.at(1).toFloat(&ok);
                 if (ok == 0 )
                 {
                     THROW_CRITICAL(trUtf8("Y position is not a numeric."));
                     return;
                 }

                 float Z= PosList.at(2).toFloat(&ok);
                 if (ok == 0 )
                 {
                     THROW_CRITICAL(trUtf8("Z position is not a numeric."));
                     return;
                 }

                 int cut_start = PosList.at(4).toInt(&ok);
                 if (ok == 0 )
                 {
                     THROW_CRITICAL(trUtf8("cut start is not a numeric."));
                     return;
                 }

                 int cut_end = PosList.at(5).toInt(&ok);
                 if (ok == 0 )
                 {
                     THROW_CRITICAL(trUtf8("cut end is not a numeric."));
                     return;
                 }

                 qDebug() << "cut_end="  << cut_end ;
                 qDebug() << "list.at(0).isEmpty()=" << list.at(0).isEmpty();


                sources1[cnt].SetPosition( harkio2::MakePositionFromVector<float>(harkio2::Vector3<float>( X,Y,Z )));

                files.clear();
                files.resize(1);
                files[0].SetName(PosList.at(3).toStdString());
                files[0].SetMicIndices(channelVector);
                sources1[cnt].SetFiles(files);
                sources1[cnt].SetCutStart(cut_start);
                sources1[cnt].SetCutEnd(cut_end);

                cnt++;
            }

        }//for end


        tsp20->SetSources(sources1);

        writer->Write(boost::dynamic_pointer_cast<harkio2::harkconfig::HarkConfigFile>(tsp20));

        out2.close();


       ::system((boost::format("tidy -o %s -xml -utf8 -w 255 -c -q -asxml -i %s ")
                               % m_convfile.c_str()
                               % m_convfile.c_str()).str().c_str());


        //display 表示
        QFile hark_config( m_convfile.c_str() );

        if (!hark_config.open(QIODevice::ReadOnly))
        {
           THROW_CRITICAL(trUtf8("Can not read tmp file"));
        }


       QTextStream inbuf(&hark_config);
       QString outbuf = inbuf.readAll();
       ui->tsp_textEdit->setStyleSheet(" color: white;font: bold 16px;");
       ui->tsp_textEdit->setText(outbuf);

       //tmp delete
       std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
       int ret = system(temp.c_str());
       qDebug() << __func__ << " system=" << ret << " temp=" << temp.c_str() ;


    }
    catch(const harkio2::AnalyzeError& e)
    {
        THROW_CRITICAL(trUtf8("Can not read xml."));
    }
    // write error
    catch (const harkio2::InvalidHarkFile& e)
    {
        THROW_CRITICAL(trUtf8("Can not write hark_config_file."));
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("Can not change xml from text."));

    }

}











//--------------------------------------------------------//
//ImpulseResponse LIST FILE XML->TEXT
//--------------------------------------------------------//
void MainWindow::IMPconvXmlToText()
{

    try
    {

        qDebug() <<  "IMPconvXmlToText start" ;

        QString text = ui->imp_textEdit->toPlainText();

        std::vector<std::vector<double> > lists; // マイク位置

        // 一時ディレクトリを生成
        m_convfile = "";
        boost::filesystem::path temp_dir = CreateTemporaryDirectory();
        m_convfile = temp_dir / "convfile_path" ;

        QFile hark_config( m_convfile.c_str());

        if (!hark_config.open(QIODevice::WriteOnly))
        {
            THROW_CRITICAL("Can not open the temporary file");
            return;
        }


        QTextStream out(&hark_config);

        out << text << flush;

        hark_config.close();


        boost::shared_ptr<harkio2::harkconfig::ImpulseResponseListFile2_0> imp_file =
            boost::dynamic_pointer_cast<harkio2::harkconfig::ImpulseResponseListFile2_0>(
                    harkio2::harkconfig::HarkConfigFileXMLReader( m_convfile.c_str()).Read());

        if (!imp_file)
        {
            THROW_CRITICAL("invalid TSPListFile");
            return;
        }

        //work画面open
        ui->stackedWidget->setCurrentIndex(DISP_scrWork);


        QString outbuf   = "# ImpulseResponse-ListFile\n";
//        outbuf  = outbuf + "# DateTime: \n";
//        outbuf  = outbuf + "ImpulseResponse-ListFile\n";
//        outbuf  = outbuf + "Version 1.0\n";


        //channel list
        QStringList list = text.split("\n");

        QString strCh;
        QString channel_list = "";


        for(int i=0 ; i< list.count() ; i++)
        {
            strCh =  list.at(i).trimmed();

            if( strCh.startsWith("</Files>") )
            {
                break;
            }
            else if( strCh.startsWith("<MicIndex>") )
            {
                strCh.replace("<MicIndex>"  , " ");
                strCh.replace("</MicIndex>" , " ");
                channel_list =  channel_list + ",";
                channel_list =  channel_list + strCh.trimmed();
            }

        }//for end


        //最初のカンマ削除
        channel_list.replace( 0 , 1 , "");
        outbuf = outbuf + "ChannelList = " + channel_list +  "\n" ;

        const auto& sources = imp_file->GetSources();

        outbuf = outbuf + "\n";
        outbuf = outbuf + "# list of sound source position and TSP file.\n";
        outbuf = outbuf + "# x, y, z , file name, IR start with potision in sample(starting with 1).\n";

        QString StrPos;
        for (size_t i = 0; i < sources.size(); ++i)
        {
            //channel番号を * に変換
            QString StrName = QString("%1").arg( QString::fromStdString( ( boost::format("%s") % imp_file->GetSources()[i].GetFiles()[0].GetName() ).str() ) );
            int indx_strat = StrName.indexOf("_ch",0);
            int indx_end   = StrName.indexOf(".flt",0);
            StrName.replace( indx_strat+3 , indx_end-(indx_strat+3),"*"  );


            StrPos = QString("%1, %2%3 , %4%5 , %6%7, %8%9, %10%11").arg( sources[i].GetPosition().GetPositionVector().GetX() ,0,'f',4 )
                                                         .arg("  ").arg( sources[i].GetPosition().GetPositionVector().GetY() ,0,'f',4 )
                                                         .arg("  ").arg( sources[i].GetPosition().GetPositionVector().GetZ() ,0,'f',4 )
                                                         .arg("  ")
                                                         .arg( StrName )
                                                         .arg("  ").arg(imp_file->GetSources()[i].GetCutStart() )
                                                         .arg("  ").arg(imp_file->GetSources()[i].GetCutEnd());

            outbuf = outbuf + StrPos + "\n";
        }


        //表示
        ui->work_textEdit->setStyleSheet(" color: white;font: bold 16px;");
        ui->work_textEdit->setText(outbuf);



         //tmp delete
         std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
         int ret = system(temp.c_str());
         qDebug() << __func__ << " system=" << ret << " temp=" << temp.c_str() ;

    }
    catch(const harkio2::AnalyzeError& e)
    {
        THROW_CRITICAL(trUtf8("Can not read XML text."));
    }
    catch (...)
    {

    }


}









//--------------------------------------------------------//
//IMP LIST FILE TEXT->XML
//--------------------------------------------------------//
void MainWindow::IMPconvTextToXml()
{

    try
    {

        // 一時ディレクトリを生成
        m_convfile = "";
        boost::filesystem::path temp_dir = CreateTemporaryDirectory();
        m_convfile = temp_dir/"convfile_path";

        std::ofstream out2( m_convfile.c_str() );
        boost::shared_ptr<harkio2::harkconfig::HarkConfigFileXMLWriter> writer(new harkio2::harkconfig::HarkConfigFileXMLWriter(out2));
        boost::shared_ptr<harkio2::harkconfig::ImpulseResponseListFile2_0> imp20(new harkio2::harkconfig::ImpulseResponseListFile2_0());


        QString work_text = ui->work_textEdit->toPlainText();
        QStringList list = work_text.split("\n");

        QString ChannelList;
        QStringList strList;

        for(int i=0;i< list.count();i++)
        {
            if( list.at(i).indexOf("flt") > 0 )
            {
                break;
            }

            if( list.at(i).startsWith("ChannelList") )
            {
                strList =  list.at(i).split("=" );
                strList = strList.at(1).split("#");
                ChannelList = strList.at(0).trimmed();
            }
        }//for end


        //channelList
        QStringList chList = ChannelList.split(",");

        vector<unsigned int> channelVector;

        int value;
        bool ok;
        for(int i=0;i<chList.size();i++)
        {
            value = chList.at(i).toInt(&ok);
            if (ok == 0 )
            {
                THROW_CRITICAL(trUtf8("ChannelList is not a numeric."));
                return;
            }

            channelVector.push_back(value);
        }


        const unsigned int nmic = chList.size();
        const harkio2::FileType::Kind sourceFileType = harkio2::FileType::FLOAT;
        const Glib::ustring comment = u8R"(created by harktool4)";

        imp20->SetNumMic(nmic);
        imp20->SetSourceFileType(sourceFileType);
        imp20->SetComment(comment);

        //Sources sizeをカウント
        int  SourcesSize = 0;
        for(int i = 0 ;i< list.count();i++)
        {
            if( list.at(i).indexOf("flt") > 0 )
            {
                SourcesSize++;
            }
        }

        std::vector<harkio2::harkconfig::Source2> sources1(SourcesSize);
        std::vector<harkio2::harkconfig::File1> files;

        //position部分を探す
        int cnt=0;
        for(int i = 0 ;i< list.count();i++)
        {
            if( list.at(i).indexOf("flt") > 0 )
            {
                QString str = list.at(i);
    //            int idx = str.indexOf("%");
    //            str = str.replace( idx ,2 , "," ); //%%をカンマに変換

                QStringList PosList = str.split(",");

                if(PosList.count() != 6 )
                {
                    THROW_CRITICAL( QString("Data is not enough.\n[%1]").arg( list.at(i).toStdString().c_str() ) ) ;
                    return;
                }


                //numeric check
                bool ok;
                float X= PosList.at(0).toFloat(&ok);

                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("X position is not a numeric."));
                    return;
                }

                float Y= PosList.at(1).toFloat(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("Y position is not a numeric."));
                    return;
                }

                float Z= PosList.at(2).toFloat(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("Z position is not a numeric."));
                    return;
                }

                int cut_start = PosList.at(4).toInt(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("cut start is not a numeric."));
                    return;
                }

                int cut_end = PosList.at(5).toInt(&ok);
                if (ok == 0 )
                {
                    THROW_CRITICAL(trUtf8("cut end is not a numeric."));
                    return;
                }


                sources1[cnt].SetPosition( harkio2::MakePositionFromVector<float>(harkio2::Vector3<float>( X ,Y ,Z )));

                files.clear();
                files.resize(nmic);

                for( unsigned int ii=0 ; ii<nmic ;ii++)
                {
                    //file name 変換
                    QString strName = PosList.at(3);

                    int idx = strName.indexOf("*");

                    //*をchannelに変換
                    strName = strName.replace( idx , 1 , ( boost::format("%02d") % channelVector.at(ii) ).str().c_str() );

                    files[ii].SetName(strName.toStdString());
                    files[ii].SetMicIndices({channelVector.at(ii)});
                }

                sources1[cnt].SetFiles(files);
                sources1[cnt].SetCutStart(cut_start);
                sources1[cnt].SetCutEnd(cut_end);

                cnt++;
            }
        }//for end


        imp20->SetSources(sources1);

        writer->Write(boost::dynamic_pointer_cast<harkio2::harkconfig::HarkConfigFile>(imp20));

        out2.close();

       ::system((boost::format("tidy -o %s -xml -utf8 -w 255 -c -q -asxml -i %s ")
                               % m_convfile.c_str()
                               % m_convfile.c_str()).str().c_str());

        //display 表示
        QFile hark_config( m_convfile.c_str() );

       if (!hark_config.open(QIODevice::ReadOnly))
       {
           THROW_CRITICAL(trUtf8("Can not read conversion file"));
           return;
       }

       QTextStream inbuf(&hark_config);
       QString outbuf = inbuf.readAll();
       ui->imp_textEdit->setStyleSheet(" color: white;font: bold 16px;");
       ui->imp_textEdit->setText(outbuf);

       //tmp delete
       std::string temp = (boost::format("rm -rf %s") % temp_dir).str();
       int ret = system(temp.c_str());
       qDebug() << __func__ << " system=" << ret << " temp=" << temp.c_str() ;

    }
    catch(const harkio2::AnalyzeError& e)
    {
        THROW_CRITICAL(trUtf8("Can not read xml."));
    }
    // write error
    catch (const harkio2::InvalidHarkFile& e)
    {
        THROW_CRITICAL(trUtf8("Can not write hark_config_file."));
    }
    catch (...)
    {
        THROW_CRITICAL(trUtf8("Can not change xml from text."));
    }
}










void MainWindow::on_action_Clear_triggered()
{
     MainWindow::on_Button_Clear_clicked();
}


void MainWindow::on_action_Plot_triggered()
{
    MainWindow::on_Button_Graph_clicked();
}

void MainWindow::on_action_Create_triggered()
{
    MainWindow::on_button_Create_clicked();
}

void MainWindow::on_action_Separation_triggered()
{
    MainWindow::on_button_Separation_clicked();
}

void MainWindow::on_action_Localization_triggered()
{
    MainWindow::on_button_Localization_clicked();
}

void MainWindow::on_action_Noise_triggered()
{
    MainWindow::on_Button_NOISE_clicked();
}

void MainWindow::on_action_MicArray_triggered()
{
    MainWindow::on_Button_MIC_clicked();
}

void MainWindow::on_action_Tsp_triggered()
{
    MainWindow::on_Button_TSP_clicked();
}

void MainWindow::on_action_Implus_triggered()
{
    MainWindow::on_Button_IMP_clicked();
}


void MainWindow::on_Button_save2_clicked()
{

     MainWindow::on_action_save_triggered();
}


//XML->Textファイル変換
void MainWindow::on_action_XML_TEXT_triggered()
{

    int index = ui->stackedWidget_2->currentIndex();

    switch(index)
    {
    case DISP_srcIMPout:
        MainWindow::IMPconvXmlToText();
        break;

    case DISP_srcTSPout:
        MainWindow::TSPconvXmlToText();
        break;
    }
}


//XML->Textファイル変換
void MainWindow::on_arrow_button_2_clicked()
{
    int index = ui->stackedWidget_2->currentIndex();

    switch(index)
    {
    case DISP_srcIMPout:
        MainWindow::IMPconvXmlToText();
        break;

    case DISP_srcTSPout:
        MainWindow::TSPconvXmlToText();
        break;
    }
}



//TEXT ->XMLファイル変換
void MainWindow::on_arrow_button_1_clicked()
{
    int index = ui->stackedWidget_2->currentIndex();

    switch(index)
    {
    case DISP_srcIMPout:
        MainWindow::IMPconvTextToXml();
        break;

    case DISP_srcTSPout:
        MainWindow::TSPconvTextToXml();
        break;
    }

}



